Ascon Pilot SDK
===============


Требования
----------

Для создания расширений к Pilot-ICE/ECM требуется один из следующих выпусков Visual Studio
- Visual Studio 2015 Professional Edition или старше
- Visual Studio 2015 Community  Edition
- Visual Studio 2013 Community Edition
- Visual Studio 2013 Express for Desktop или Express for Web (with Update 2 или старше)
- Visual Studio 2013 Professional или старше



Создание и настройка проекта расширения
---------------------------------------

Чтобы создать новое расширение для системы Pilot выполните следующие действия:

- Создайте проект типа ClassLibrary;
- Вызовите диалог настройки проекта. В поле AssemblyName добавьте к названию сборки `.ext` Например `WPFPlugin.ext`
- Установите целевую платформу (Target Framework) в .NET Framework 4 Client Profile;
- Вызовите диалог редактирования информации о сборке. (Assembly Information...) Для этого нажмите на кнопку `Assembly Information...` и заполните необходимые поля.
    > Обязательно заполните поля Title, Product и Assembly version. Т.к. клиент при загрузке вашего расширения будет искать информацию именно в этих полях.
- Подключите к проекту ссылку на .NET сборку System.ComponentModel.Composition.

Для удобства отладки проекта расширения выполните следующие шаги:
- настройте сборку проекта в следующую папку
`C:\Users\<current_user>\AppData\Local\ASCON\Pilot-ICE\Development ` для систем Windows 7 и старше
`C:\Documents and Settings\<current_user>\Local Settings\Application Data\ASCON\Pilot-ICE\Development` для Windows XP
- на вкладке *Debug* выберите пункт *Start external program* и введите путь до установленного клиента Pilot-ICE/ECM.
- сохраните изменения и запустите проект на исполнение.

> Папка Development по умолчанию отсутствует. Ее необходимо создать.



Подключение SDK к проекту расширения
------------------------------------

Для подключения SDK Pilot к проекту расширения можно воспользоваться встроенным в Visual Studio механизмом распространения пакетов - NuGet Packages Manager.
Для того, чтобы подключить SDK с помощью Nuget Package Manager выполните следующие шаги:
- Вызовите [Package Manager Console](https://docs.nuget.org/docs/start-here/using-the-package-manager-console);
- Выполните команду:
```cpp
Install-Package Ascon.Pilot.SDK
```

- Щелкните правой кнопкой мыши на проекте и вызовите команду `Manage NuGet Packages`;
- Установите NuGet пакет Ascon.Pilot.SDK для вашего проекта.

Для того, чтобы подключить SDK вручную выполните следующие шаги:
- Скачайте примеры использования SDK из [центра загрузок Pilot-ICE](http://pilot.sd.ascon.ru/);
- Распакуйте загруженный архив в любую папку.
- Настройте NuGet Package Manager на работу с папкой SDK из загруженного архива.
- Установите NuGet пакет Ascon.Pilot.SDK для вашего проекта.


Отладка и логирование ошибок расширений
---------------------------------------

Для ведения журнала подключений MEF в Ascon.Pilot.PilotICE.exe.config нужно дописать строчки:

```xml
<system.diagnostics>
    <sources>
        <source name="System.ComponentModel.Composition" switchValue="All">
            <listeners>
                <add name="fileListener"
                     type="System.Diagnostics.TextWriterTraceListener"
                     initializeDataReadOnlyCollection<Guid> ChildrenReadOnlyCollection<Guid> Children="composition.log" />
            </listeners>
        </source>
    </sources>
  <trace autoflush="true" indentsize="4" />
</system.diagnostics>
```

## Содержание
[Возможные типы расширений](#typesOfExtensions)
- [Новая вкладка INewTabPage](#INewTabPage)
- [Панель инструментов IToolbar](#IToolbar)
	- [IToolbarBuilder](#IToolbarBuilder)
	- [IToolbarButtonItemBuilder](#IToolbarButtonItemBuilder)
	- [IToolbarToggleButtonItemBuilder](#IToolbarToggleButtonItemBuilder)
	- [IToolbarMenuButtonItemBuilder](#IToolbarMenuButtonItemBuilder)
	- [IToolbarItemSubmenuHandler](#IToolbarItemSubmenuHandler)
- [Меню IMenu](#IMenu)
	- [IMenuBuilder](#IMenuBuilder)
	- [IMenuItemBuilder](#IMenuItemBuilder)
- [Объекты контекста для построения меню и панели инструментов](#Context)
	- [TasksViewContext](#TasksViewContext)
	- [ObjectsViewContext](#ObjectsViewContext)
	- [MainViewContext](#MainViewContext)
	- [SystemTrayContext](#SystemTrayContext)
	- [StorageContext](#StorageContext)
    - [XpsMergerContext](#XpsMergerContext)
    - [XpsRenderContext](#XpsRenderContext)
    - [DocumentFilesContext](#DocumentFilesContext)
	- [LinkedTasksContext](#LinkedTasksContext)
	- [LinkedObjectsContext](#LinkedObjectsContext)
- [Перехват автоимпорта документов IAutoimportHandler](#IAutoimportHandler)
- [Перехват печати на принтер IPrintHandler](#IPrintHandler)
- [Перехват отображения атрибутов в карточке объекта IObjectCardHandler](#IObjectCardHandler)
	- [Редактирования атрибутов в карточке IAttributeModifier](#IAttributeModifier)
	- [Контекст карточки ObjectCardContext](#ObjectCardContext)
- [Устаревшие интерфейсы](#OldInterfaces)
	- [Контекстное меню обозревателя проектов IObjectContextMenu](#IObjectContextMenu)
		- [IMenuHost](#IMenuHost)
	- [Контекстное меню заданий ITaskContextMenu](#ITaskContextMenu)
	- [Главное меню IMainMenu](#IMainMenu)
	- [Системное меню ISystemTrayMenu](#ISystemTrayMenu)
	- [Контекстное меню Pilot-Storage IStorageContextMenu](#IStorageContextMenu)

[Интерфейсы управления данными](#DataManagementInterfaces)
- [IObjectsRepository](#IObjectsRepository)
- [IObjectModifier](#IObjectModifier)
	- [IObjectBuilder](#IObjectBuilder)
- [ITaskModifier](#ITaskModifier)
	- [ITaskBuilder](#ITaskBuilder)
- [IFileProvider](#IFileProvider)

[Интерфейсы работы с данными](#DataProccessingInterfaces)
- [IXpsDigitalSigner](#IXpsDigitalSigner)
- [IXpsRender](#IXpsRender)
- [IXpsViewer](#IXpsViewer)
- [IXpsMerger](#IXpsMerger)
- [IXpsMergerSaveListener](#IXpsMergerSaveListener)
- [IGraphicLayerElement](#IGraphicLayerElement)
- [GraphicLayerElementConstants](#GraphicLayerElementConstants)

[Интерфейсы взаимодействия с клиентом Pilot-ICE/ECM](#InterfacesToTheClient)
- [ITabServiceProvider](#ITabServiceProvider)
- [IPilotDialogService](#IPilotDialogService)
- [IEventAggregator](#IEventAggregator)
- [IConnectionStateProvider](#IConnectionStateProvider)

[Работа с конфигурацией атрибутов: форматирование, нумераторы, справочники](#AttributeFormatInterfaces)
- [Интерфейс IAttributeFormatParser](#IAttributeFormatParser)
- [Интерфейс IReferenceBookConfiguration](#IReferenceBookConfiguration)
	- [Перечисление RefBookKind](#RefBookKind)
	- [Перечисление RefBookEditorType](#RefBookEditorType)
- [Интерфейс INumeratorInfo](#INumeratorInfo)

[Управление общими настройками](#CommonSettings)
- [Интерфейс ISettingsFeature](#ISettingsFeature)
- [Интерфейс IPersonalSettings](#IPersonalSettings)
- [Интерфейс ISettingValueProvider](#ISettingValueProvider)
- [Класс SystemSettingsKeys](#SystemSettingsKeys)

[Поиск](#SearchInterfaces)
- [Интерфейс ISearchService](#ISearchService#ISearchService)
- [Интерфейс ISmartFolderQueryBuilder](#ISmartFolderQueryBuilder)
- [Пример создания умной папки](#SmartFolderCreationExample)
- [Интерфейс IQueryBuilder](#IQueryBuilder)
- [Интерфейс ISearchResult](#ISearchResult)
- [Предустановленные условия поиска](#MainSearchTerms)
	- [Условия поиска для объектов ObjectFields](#ObjectFields)
	- [Условия поиска для заданий TaskFields](#TaskFields)
	- [Условия поиска по произвольным атрибутам AttributeFields](#AttributeFields)
	- [Общие условия поиска](#AllTerms)
- [Пример создания поискового запроса](#ObjectSearchExample)

[Объекты](#ObjectInterfaces)
- [IDataObject](#IDataObject)
- [ITaskObject](#ITaskObject)
- [IWorkflowObject](#IWorkflowObject)
- [IStageObject](#IStageObject)
- [IType](#IType)
- [IAttribute](#IAttribute)
- [IFile](#IFile)
- [ISignature](#ISignature)
- [IOrganisationUnit](#IOrganisationUnit)
- [IPerson](#IPerson)
- [IPosition](#IPosition)
- [IAccess](#IAccess)
- [IAccessRecord](#IAccessRecord)
- [IStorageDataObject](#IStorageDataObject)
- [IStateInfo](#IStateInfo)
- [IRelation](#IRelation)
- [IFilesSnapshot](#IFilesSnapshot)

[Перечисления](#Enums)
- [DataState](#DataState)
- [TaskState](#State)
- [SynchronizationState](#SynchronizationState)
- [TypeKind](#TypeKind)
- [AccessLevel](#AccessLevel)
- [StorageObjectState](#StorageObjectState)
- [ObjectRelationType](#ObjectRelationType)
- [TaskKind](#TaskKind)


[События](#Events)
- [OfflineEventArgs](#OfflineEventArgs)
- [OnlineEventArgs](#OnlineEventArgs)
- [CloseTabEventArgs](#CloseTabEventArgs)
- [OpenTabEventArgs](#OpenTabEventArgs)
- [LoadedEventArgs](#LoadedEventArgs)
- [UnloadedEventArgs](#UnloadedEventArgs)

[Дополнительно](#Additionally)
- [Пример создания объекта с нумератором](#NumeratorExample)

<a name="typesOfExtensions"/>
Возможные типы расширений
-------------------------

Система расширений Pilot-ICE/ECM основана на стандартном механизме расширений приложений в .NET - Managed Extensibility Framework (MEF). SDK содержит ряд экспортируемых и импортируемых интерфейсов. К экспортируемым интерфейсам относятся интерфейсы, которые описывают тип расширения. К импортируемым интерфейсам относятся интерфейсы взаимодействия с клиентом и работы с данными.

<a name="INewTabPage"/>
### Интерфейс INewTabPage ###

Для того, чтобы встроить расширение в новую вкладку главного окна клиента Pilot-ICE/ECM необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `INewTabPage`. Обязательно пометить класс, реализующий интерфейс **INewTabPage** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;
namespace WpfExtension
{
    [Export(typeof(INewTabPage))]
    public class MainModule : INewTabPage
    {
        public void BuildNewTabPage(INewTabPageHost host)
        {
        }

        public void OnButtonClick(string name)
        {
        }
    }
}
```

##### INewTabPage.BuildNewTabPage #####

Метод вызывается перед тем, как будет построена вкладка *Новая вкладка*.

```cpp
void INewTabPage.BuildNewTabPage(INewTabPageHost host);
```
где:
 - `host` - интерфейс, позволяющий добавлять кнопки.

Интерфейс содержит только один метод `AddButton`
```cpp
void INewTabPageHost.AddButton(string title, string name, string toolTip, byte[] svgIcon);
```
где:
 - `title` - локализованное название кнопки;
 - `name` - уникальное имя кнопки в пределах расширения;
 - `toolTip` - локализованная подсказка, появляющаяся при наведении курсора мыши на кнопку;
 - `svgIcon` - массив байт SVG иконки. Клиент поддерживает иконки только в svg формате.


##### INewTabPage.OnButtonClick #####

Метод вызывается при нажатии на кнопку, созданную расширением в методе `BuildNewTabPage`.

```cpp
void INewTabPage.OnButtonClick(string name);
```
где:
 - `name` - уникальное имя кнопки, по которой произошло нажатие.




<a name="IToolbar"/>
### Интерфейс `IToolbar<TToolbarContext>` ###

Интерфейс позволяет встраивать новые команды в панель инструментов. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IToolbar<TToolbarContext>` и обязательно пометить класс, реализующий интерфейс **IToolbar< TToolbarContext >** атрибутом `[Export]`.

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IToolbar<TasksViewContext>))]
    public class ToolbarSample : IToolbar<TasksViewContext>
    {
        public void Build(IToolbarBuilder builder, TasksViewContext context)
        {
         	//...
        }

        public void OnToolbarItemClick(string name, TasksViewContext context)
        {
         	//...
        }
    }
}
```

Чтобы встроить новые команды в панель инструментов вкладки *Задания* необходимо реализовать интерфейc `IToolbar<TasksViewContext>`.

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IToolbar<TasksViewContext>))]
    public class ToolbarSample : IToolbar<TasksViewContext>
    {
        ...
    }
}
```

Чтобы встроить новые команды в панель инструментов *Обозревателя документов* необходимо реализовать интерфейc `IToolbar<ObjectsViewContext>`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IToolbar<ObjectsViewContext>))]
    public class ToolbarSample : IToolbar<ObjectsViewContext>
    {
        ...
    }
}
```


#### `IToolbar<TasksViewContext>.Build` ####

Метод вызывается перед построением панели инструментов.

```cpp
void IToolbar<TToolbarContext>.Build(IToolbarBuilder builder, TToolbarContext context);
```
где:
 - `builder` - интерфейс, позволяющий добавлять различные элементы в панель инструментов;
 - `context` - контекст, позволяющий получить дополнительные сведения о селектированных элементах.


#### `IToolbar<TToolbarContext>.OnToolbarItemClick` ####

Метод вызывается при нажатии на элемент панели инструментов.

```cpp
void IToolbar<TToolbarContext>.OnToolbarItemClick(string name, TToolbarContext context);
```
где:
 - `name` - уникальное внутреннее имя элемента панели инструментов;
 - `context` - контекст выполнения комманды.


<a name="IToolbarBuilder"/>
### Интерфейс `IToolbarBuilder` ###

Интерфейс позволяет управлять элементами панели инструментов.

#### `IToolbarBuilder.ItemNames` ####

Свойство возвращает список уникальных имен элементов панели инструментов.
```cpp
IEnumerable<string> IToolbarBuilder.ItemNames { get; }
```

#### `IToolbarBuilder.Count` ####

Свойство возвращает количество элементов панели инструментов.
```cpp
int IToolbarBuilder.Count { get; }
```

#### `IToolbarBuilder.AddSeparator` ####

Метод позволяет добавить разделитель в панель инструментов.
```cpp
void IToolbarBuilder.AddSeparator(int index);
```
где:
 - `index` - индекс, куда вставить разделитель.


#### `IToolbarBuilder.AddButtonItem` ####

Метод позволяет добавить кнопку в панель инструментов.
```cpp
IToolbarButtonItemBuilder AddButtonItem(string name, int index);
```
где:
 - `name` - уникальное внутреннее имя кнопки;
 - `index` - индекс, куда вставить кнопку.

Возвращает интерфейс добавления свойств кнопки `IToolbarButtonItemBuilder`.


#### `IToolbarBuilder.AddMenuButtonItem` ####

Метод позволяет добавить кнопку с выпадающим меню в панель инструментов.
```cpp
IToolbarMenuButtonItemBuilder AddMenuButtonItem(string name, int index);
```
где:
 - `name` - уникальное внутреннее имя кнопки;
 - `index` - индекс, куда вставить кнопку.

Возвращает интерфейс добавления свойств кнопки `IToolbarMenuButtonItemBuilder`.


#### `IToolbarBuilder.AddToggleButtonItem` ####

Метод позволяет добавить кнопку-переключатель в панель инструментов.
```cpp
IToolbarToggleButtonItemBuilder AddToggleButtonItem(string name, int index);
```
где:
 - `name` - уникальное внутреннее имя кнопки;
 - `index` - индекс, куда вставить кнопку.

Возвращает интерфейс добавления свойств кнопки `IToolbarToggleButtonItemBuilder`.



#### `IToolbarBuilder.ReplaceButtonItem` ####

Метод позволяет заменить любой элемент панели инструментов на кнопку.
```cpp
IToolbarButtonItemBuilder ReplaceButtonItem(string name);
```
где:
 - `name` - уникальное внутреннее имя элемента, который будет заменен.

Возвращает интерфейс добавления свойств кнопки `IToolbarButtonItemBuilder`.



#### `IToolbarBuilder.ReplaceMenuButtonItem` ####

Метод позволяет заменить любой элемент панели инструментов на кнопку с выподающим меню.
```cpp
IToolbarMenuButtonItemBuilder ReplaceMenuButtonItem(string name);
```
где:
 - `name` - уникальное внутреннее имя элемента, который будет заменен.

Возвращает интерфейс добавления свойств кнопки `IToolbarMenuButtonItemBuilder`.



#### `IToolbarBuilder.ReplaceToggleButtonItem` ####

Метод позволяет заменить любой элемент панели инструментов на кнопку-переключатель.
```cpp
IToolbarToggleButtonItemBuilder ReplaceToggleButtonItem(string name);
```
где:
 - `name` - уникальное внутреннее имя элемента, который будет заменен.

Возвращает интерфейс добавления свойств кнопки `IToolbarToggleButtonItemBuilder`.


<a name="IToolbarButtonItemBuilder"/>
### Интерфейс `IToolbarButtonItemBuilder` ###

Интерфейс позволяет добавить свойства к элементу панели инструменотов.


#### `IToolbarButtonItemBuilder.WithHeader` ####

Метод позволяет добавить отображаемое в UI название.
```cpp
IToolbarButtonItemBuilder WithHeader(string header);
```
где:
 - `header` - отображаемое в UI название элемента.


#### `IToolbarButtonItemBuilder.WithIcon` ####

Метод позволяет добавить иконку элементу панели инструментов. Обратите внимание, иконка должна быть в формате `SVG`
```cpp
IToolbarButtonItemBuilder WithIcon(byte[] svgIcon);
```
где:
 - `svgIcon` - иконка в формате SVG.


#### `IToolbarButtonItemBuilder.WithIsEnabled` ####

Метод позволяет задать значение доступности элемента.
```cpp
IToolbarButtonItemBuilder WithIsEnabled(bool value);
```


#### `IToolbarButtonItemBuilder.WithShowHeader` ####

Метод позваляет спрятать или показать заголовок элемента. Чтобы показать элемент только в виде иконки необходимо в метод `WithShowHeader` передать параметр `false`
```cpp
IToolbarButtonItemBuilder WithShowHeader(bool value);
```


#### `IToolbarButtonItemBuilder.WithHint` ####

Метод позволяет добавить подсказку к элементу панели инструментов.
```cpp
IToolbarButtonItemBuilder WithHint(string hint);
```
где:
 - `hint` - текст подсказки.


<a name="IToolbarToggleButtonItemBuilder"/>
### Интерфейс `IToolbarToggleButtonItemBuilder` ###

Интерфейс позволяет добавить свойства к кнопке-переключателю панели инструменотов. Этот интерфейс является наследником `IToolbarButtonItemBuilder`.

#### `IToolbarToggleButtonItemBuilder.WithIsChecked` ####

Метод позволяет задать состояние кнопки-переключателя.
```cpp
IToolbarToggleButtonItemBuilder WithIsChecked(bool value);
```


<a name="IToolbarMenuButtonItemBuilder"/>
### Интерфейс `IToolbarMenuButtonItemBuilder` ###

Интерфейс позволяет добавить свойства к кнопке с выпадающим меню панели инструменотов. Этот интерфейс является наследником `IToolbarButtonItemBuilder`.

#### `IToolbarMenuButtonItemBuilder.WithMenu` ####

Метод позволяет добавить контекстное меню к кнопке.
```cpp
IToolbarMenuButtonItemBuilder WithMenu(IToolbarItemSubmenuHandler itemSubmenuHandler);
```
где:
 - `itemSubmenuHandler` - класс описывающий построение выпадающего меню.


<a name="IToolbarItemSubmenuHandler"/>
### Интерфейс `IToolbarItemSubmenuHandler` ###

Интерфейс построения меню для кнопки панели инструментов. В пакет `SDK` уже поставляется класс реализующий данный интерфейс - `ToolbarItemSubmenuHandler`. Для того, чтобы построить меню необходимо унаследоваться от этого класса.

#### `IToolbarItemSubmenuHandler.OnSubmenuRequested` ####

Метод вызывается перед тем как появится меню у кнопки панели инструментов.
```cpp
void OnSubmenuRequested(IToolbarBuilder builder);
```
где:
 - `builder` - интерфейс построения меню.


Пример построения меню для кнопки в панели инструментов:
```cpp
[Export(typeof(IToolbar<TasksViewContext>))]
public class ToolbarSample : IToolbar<TasksViewContext>
{
	public void Build(IToolbarBuilder builder, TasksViewContext context)
    {
    	builder.AddMenuButtonItem("tbsMenuButton", 0)
                .WithMenu(new MenuHandler())
                .WithHeader("Menu button")
                .WithHint("Menu button");
    }

	public void OnToolbarItemClick(string name, TasksViewContext context)
    {
    	if (name == "MenuName")
        {
        	//do somethig
        }
    }
}

public class MenuHandler : ToolbarItemSubmenuHandler
{
    public override void OnSubmenuRequested(IToolbarBuilder builder)
    {
        builder.AddButtonItem("MenuName", 0).WithHeader("item");
    }
}

```

Подробнее можно посмотреть в примере `Ascon.Pilot.SDK.ToolbarSample`, поставляемом вместе с пакетом `SDK`.



<a name="IMenu"/>
### Интерфейс `IMenu<TMenuContext>` ###

Интерфейс позволяет встраивать новые команды в различные меню приложения в зависимости от контекста. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IMenu<TMenuContext>` и обязательно пометить класс, реализующий интерфейс **IMenu< TMenuContext >** атрибутом `[Export]`.

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IMenu<TasksViewContext>))]
    public class ToolbarSample : IMenu<TasksViewContext>
    {
        public void Build(IMenuBuilder builder, TasksViewContext context)
        {
         	//...
        }

        public void OnMenuItemClick(string name, TasksViewContext context)
        {
         	//...
        }
    }
}
```

Чтобы встроить новые команды в контекстное меню вкладки *Задания* необходимо реализовать интерфейc `IMenu<TasksViewContext>`.

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IMenu<TasksViewContext>))]
    public class TasksContextMenuSample : IMenu<TasksViewContext>
    {
        ...
    }
}
```

Чтобы встроить новые команды в контекстное меню *Обозревателя документов* необходимо реализовать интерфейc `IMenu<ObjectsViewContext>`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IMenu<ObjectsViewContext>))]
    public class ObjectsContextMenuSample : IMenu<ObjectsViewContext>
    {
        ...
    }
}
```


#### `IMenu<TMenuContext>.Build` ####

Метод вызывается перед построением контекстного меню.

```cpp
void IMenu<TMenuContext>.Build(IMenuBuilder builder, TMenuContext context);
```
где:
 - `builder` - интерфейс, позволяющий добавлять различные элементы в меню;
 - `context` - контекст, позволяющий получить дополнительные сведения о селектированных элементах.


#### `IMenu<TMenuContext>.OnMenuItemClick` ####

Метод вызывается при нажатии на элемент меню.

```cpp
void IMenu<TMenuContext>.OnMenuItemClick(string name, TMenuContext context);
```
где:
 - `name` - уникальное внутреннее имя элемента меню;
 - `context` - контекст выполнения комманды.


<a name="IMenuBuilder"/>
### Интерфейс `IMenuBuilder` ###

Интерфейс позволяет управлять элементами меню.

#### `IMenuBuilder.ItemNames` ####

Свойство возвращает список уникальных имен элементов меню.
```cpp
IEnumerable<string> IMenuBuilder.ItemNames { get; }
```

#### `IMenuBuilder.Count` ####

Свойство возвращает количество элементов меню.
```cpp
int IMenuBuilder.Count { get; }
```

#### `IMenuBuilder.AddSeparator` ####

Метод позволяет добавить разделитель в меню.
```cpp
void IMenuBuilder.AddSeparator(int index);
```
где:
 - `index` - индекс, куда вставить разделитель.


#### `IMenuBuilder.AddItem` ####

Метод позволяет добавить пункт меню.
```cpp
IMenuItemBuilder AddItem(string name, int index);
```
где:
 - `name` - уникальное внутреннее имя пункта меню;
 - `index` - индекс, куда вставить пункт меню.

Возвращает интерфейс добавления свойств пункта меню `IMenuItemBuilder`.


#### `IMenuBuilder.ReplaceItem` ####

Метод позволяет заменить любой пункт меню.
```cpp
IMenuItemBuilder ReplaceItem(string name);
```
где:
 - `name` - уникальное внутреннее имя;


#### `IMenuBuilder.GetItem` ####

Метод позволяет получить построитель пункта меню.
```cpp
IMenuBuilder GetItem(string name);
```
где:
 - `name` - уникальное внутреннее имя;

Возвращает интерфейс добавления пунктов меню `IMenuBuilder`.

<a name="IMenuItemBuilder"/>
### Интерфейс `IMenuItemBuilder` ###

Интерфейс позволяет добавлять свойства пункту меню.

#### `IMenuItemBuilder.WithHeader` ####

Метод позволяет добавить отображаемое в UI название.
```cpp
IMenuItemBuilder WithHeader(string header);
```
где:
 - `header` - отображаемое в UI название пункта меню.


#### `IMenuItemBuilder.WithIcon` ####

Метод позволяет добавить иконку. Обратите внимание, иконка должна быть в формате `SVG` для всех меню кроме IMenu<StorageContext>. Иконки для меню Pilot-Storage дожны быть в формате `PNG`.

```cpp
IMenuItemBuilder WithIcon(byte[] icon);
```
где:
 - `icon` - иконка.


#### `IMenuItemBuilder.WithIsEnabled` ####

Метод позволяет задать значение доступности элемента.
```cpp
IMenuItemBuilder WithIsEnabled(bool value);
```

#### `IMenuItemBuilder.WithSubmenu` ####

Метод позволяет добавить подменю к текущему пункту меню.
```cpp
IMenuBuilder WithSubmenu();
```

<a name="Context"/>
###Объекты контекста для построения меню и панели инструментов###

<a name="TasksViewContext"/>
####TasksViewContext####
Чтобы встроить расширение в контекстное меню или панель инструментов вкладки *Задания* используйте этот тип контекста:
```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<TasksViewContext>))]
    public class ContextMenuSample : IMenu<TasksViewContext>
    {
        ...
    }
}
```

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IToolbar<TasksViewContext>))]
    public class ToolbarSample : IToolbar<TasksViewContext>
    {
        ...
    }
}
```

<a name="ObjectsViewContext"/>
####ObjectsViewContext####
Чтобы встроить расширение в контекстное меню или панель инструментов *Обозревателя проектов* используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<ObjectsViewContext>))]
    public class ContextMenuSample : IMenu<ObjectsViewContext>
    {
        ...
    }
}
```

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IToolbar<ObjectsViewContext>))]
    public class ToolbarSample : IToolbar<ObjectsViewContext>
    {
        ...
    }
}
```


<a name="MainViewContext"/>
####MainViewContext####
Чтобы встроить расширение в главное меню приложения используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<MainViewContext>))]
    public class MenuSample : IMenu<MainViewContext>
    {
        ...
    }
}
```

<a name="SystemTrayContext"/>
####SystemTrayContext####
Чтобы встроить расширение в системное меню используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<SystemTrayContext>))]
    public class MenuSample : IMenu<SystemTrayContext>
    {
        ...
    }
}
```

<a name="StorageContext"/>
####StorageContext####
Чтобы встроить расширение в контекстное меню Pilot-Storage используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<StorageContext>))]
    public class MenuSample : IMenu<StorageContext>
    {
        ...
    }
}
```

<a name="XpsMergerContext"/>
####XpsMergerContext####
Чтобы встроить расширение в панель инструментов компоновщика XPS используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IToolbar<XpsMergerContext>))]
    public class XpsMergerToolbarSample : IToolbar<XpsMergerContext>
    {
        ...
    }
}
```

<a name="XpsRenderContext"/>
####XpsRenderContext####
XpsRenderContext содержит информацию об открытом в просмотрщике документе. Чтобы встроить расширение в панель инструментов просмотрщика документов используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IToolbar<XpsRenderContext>))]
    public class XpsRenderToolbarSample : IToolbar<XpsRenderContext>
    {
        ...
    }
}
```

<a name="GraphicLayerElementContext"/>
####GraphicLayerElementContext####
Графический слой позволяет встраивать через SDK растровые изображения и блоки XAML разметки в XPS документ (см. Ascon.Pilot.SDK.GraphicLayerSample). Добавленные элементы графического слоя можно перемещать по документу и вызывать контекстное меню с командами, добавленными из плагина. Графический слой не является частью XPS документа и доступен для просмотра только в Pilot. Графический слой вшивается в XPS документ только во время печати. Чтобы получить XPS с вшитым графическим слоем, необходимо отправить документ на печать в виртуальный принтер "Pilot XPS". После этого элементы графического слоя будет невозможно перемещать по документу, вызывать контекстное меню и менять содержимое через плагин. Чтобы встроить расширение в контекстное меню графического элемента используйте этот тип контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<GraphicLayerElementContext>))]
    public class GraphicLayerContextMenuSample : IMenu<GraphicLayerElementContext>
    {
        ...
    }
}
```

<a name="DocumentFilesContext"/>
####DocumentFilesContext####
Чтобы встроить расширение в контекстное меню панели файлов ECM документа используйте данный вид контекста.
Описание свойств контекста:
 - IEnumerable<IDataObject> EcmDocument { get; } - ECM документ, для которого была показана панель файлов
 - IEnumerable<IStorageDataObject> SelectedObjects { get; } - список выбранных файлов
 - bool IsContext { get; } - флаг, показывающий в каком месте был сделан вызов меню
 - bool IsMounted { get; } - флаг, определяющий был ли смонтирован на Pilot-Storage ECM документ в момент вызова меню
 - DocumentFilesUsage Usage { get; } - определяет, в каком UI используется панель файлов
 
 ### Перечисление DocumentFilesUsage ###
Перечисление UI использующих панель файлов

  ##### DocumentFilesUsage.CreateNew #####
UI создания нового ECM документа

  ##### DocumentFilesUsage.ObjectsExplorer #####
UI обозревателя документов

  ##### DocumentFilesUsage.CardView #####
UI карточки документа

  ##### DocumentFilesUsage.TemplatesManager #####
UI менеджера шаблонов документов
	
```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<DocumentFilesContext>))]
    public class MenuSample : IMenu<DocumentFilesContext>
    {
        ...
    }
}
```

<a name="LinkedTasksContext"/>
####LinkedTasksContext####
Чтобы встроить расширение в контекстное меню связанных заданий используйте данный вид контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<LinkedTasksContext>))]
    public class MenuSample : IMenu<LinkedTasksContext>
    {
        ...
    }
}
```

<a name="LinkedObjectsContext"/>
####LinkedObjectsContext####
Чтобы встроить расширение в контекстное меню вкладки "Связи" используйте данный вид контекста:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Sample
{
    [Export(typeof(IMenu<LinkedObjectsContext>))]
    public class MenuSample : IMenu<LinkedObjectsContext>
    {
        ...
    }
}
```

<a name="OldInterfaces"/>
Устаревшие интерфейсы
-------------------------

<a name="IObjectContextMenu"/>
### Интерфейс IObjectContextMenu ###

Интерфейс позволяет встраивать расширение в контекстное меню обозревателя проектов. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IObjectContextMenu` и обязательно пометить класс, реализующий интерфейс **IObjectContextMenu** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IObjectContextMenu))]
    public class ContextMenuPlugin : IObjectContextMenu
    {
        public void BuildContextMenu(IMenuHost menuHost, IEnumerable<IDataObject> selection, bool isContext)
        {
        }

        public void OnMenuItemClick(string itemName)
        {
        }
    }
}
```


##### IObjectContextMenu.BuildContextMenu #####

Метод вызывается перед построением контекстного меню *Обозревателя проектов*.
```cpp
void IObjectContextMenu.BuildContextMenu(IMenuHost menuHost, IEnumerable<IDataObject> selection, bool isContext);
```
где:
 - `menuHost` - интерфейс, позволяющий добавлять различные элементы меню;
 - `selection` - список выбранных элементов;
 - `isContext` - флаг показывающий в каком месте был сделан вызов контекстного меню:
 	+ `true` - вызов был сделан в пустой области обозревателя;
 	+ `false` - вызов был сделан на выбранном элементе.



##### IObjectContextMenu.OnMenuItemClick #####

Метод вызывается при нажатии на пункт контекстного меню.

```cpp
void IObjectContextMenu.OnMenuItemClick(string itemName);
```
где:
 - `itemName` - уникальное название пункта контекстного меню.


<a name="IMenuHost"/>
### Интерфейс IMenuHost ###

Интерфейс обеспечивает возможность управлять пунктами меню клиента.


##### IMenuHost.GetItems #####

Метод возвращает список уникальных имен пунктов контекстного меню.

```cpp
IEnumerable<string> IMenuHost.GetItems(string parentName = null);
```
где:
 - `parentName` - имя пункта родительского меню.


##### IMenuHost.AddItem #####

Метод позволяет добавлять в текущее меню свой пункт меню.

```cpp
void IMenuHost.AddItem(string name, string displayName, byte[] svgIcon, int index);
```
где:
 - `name` - имя пункта меню.
 - `displayName` - локализованное имя пункта меню.
 - `svgIcon` - массив байт иконки в формате svg.
 - `index` - индекс, куда необходимо вставить свой пункт меню.


##### IMenuHost.AddSubItem #####

Метод позволяет добавлять подменю в заданный пункт меню.

```cpp
void IMenuHost.AddSubItem(string parentName, string name, string displayName, byte[] svgIcon, int index);
```
где:
 - `parentName` - имя родительского пункта меню.
 - `name` - имя пункта меню.
 - `displayName` - локализованное имя пункта меню.
 - `svgIcon` - массив байт иконки в формате svg.
 - `index` - индекс, куда необходимо вставить свой пункт меню.


<a name="ITaskContextMenu"/>
### Интерфейс ITaskContextMenu ###

Интерфейс позволяет встроить расширение в контекстное меню обозревателя заданий. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `ITaskContextMenu` и обязательно пометить класс, реализующий интерфейс **ITaskContextMenu** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(ITaskContextMenu))]
    public class TaskContextMenuPlugin : ITaskContextMenu
    {
        public void BuildContextMenu(IMenuHost menuHost, IEnumerable<ITaskObject> selection)
        {
        }

        public void OnMenuItemClick(string itemName)
        {
        }
    }
}
```


##### ITaskContextMenu.BuildContextMenu #####

Метод вызывается перед построением контекстного меню управления заданиями.
```cpp
void ITaskContextMenu.BuildContextMenu(IMenuHost menuHost, IEnumerable<ITaskObject> selection);
```
где:
 - `menuHost` - интерфейс, позволяющий добавлять различные элементы меню.
 - `selection` - список выбранных заданий.


##### ITaskContextMenu.OnMenuItemClick #####

Метод вызывается при нажатии на пункт контекстного меню.

```cpp
void ITaskContextMenu.OnMenuItemClick(string itemName);
```
где:
 - `itemName` - уникальное название пункта контекстного меню.




<a name="IMainMenu"/>
### Интерфейс IMainMenu ###

Интерфейс позволяет встроить расширение в главное меню клиента Pilot-ICE/ECM. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IMainMenu` и обязательно пометить класс, реализующий интерфейс **IMainMenu** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace MainMenuSample
{
    [Export(typeof(IMainMenu))]
    public class MainMenuPlugin : IMainMenu
    {
        public void BuildMenu(IMenuHost menuHost)
        {
        }

        public void OnMenuItemClick(string itemName)
        {
        }
    }
}
```


##### IMainMenu.BuildContextMenu #####

Метод вызывается перед построением главного меню.

```cpp
void IMainMenu.BuildContextMenu(IMenuHost menuHost);
```
где:
 - `menuHost` - интерфейс, позволяющий добавлять различные элементы меню.


##### IMainMenu.OnMenuItemClick #####

Метод вызывается при нажатии на пункт контекстного меню.

```cpp
void IMainMenu.OnMenuItemClick(string itemName);
```
где:
 - `itemName` - уникальное название пункта меню



<a name="ISystemTrayMenu"/>
### Интерфейс ISystemTrayMenu ###

Интерфейс позволяет встроить расширение в системное меню клиента Pilot-ICE/ECM. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `ISystemTrayMenu` и обязательно пометить класс, реализующий интерфейс **ISystemTrayMenu** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace Ascon.Pilot.SDK.SystemTrayMenuSample
{
    [Export(typeof(ISystemTrayMenu))]
    public class SystemTrayMenuPlugin : ISystemTrayMenu
    {
        public void BuildMenu(IMenuHost menuHost)
        {
        }

        public void OnMenuItemClick(string itemName)
        {
        }
    }
}

```

##### ISystemTrayMenu.BuildContextMenu #####

Метод вызывается перед построением системного меню.

```cpp
void ISystemTrayMenu.BuildContextMenu(IMenuHost menuHost);
```
где:
 - `menuHost` - интерфейс, позволяющий добавлять различные элементы меню.



##### ISystemTrayMenu.OnMenuItemClick #####

Метод вызывается при нажатии на пункт системного меню.

```cpp
void ISystemTrayMenu.OnMenuItemClick(string itemName);
```
где:
 - `itemName` - уникальное название пункта системного меню



<a name="IStorageContextMenu"/>
### Интерфейс IStorageContextMenu ###

Интерфейс позволяет встраивать расширение в контекстное меню проводника Windows. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IStorageContextMenu` и обязательно пометить класс, реализующий интерфейс **IStorageContextMenu** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IStorageContextMenu))]
    public class ContextMenuPlugin : IStorageContextMenu
    {
        public void BuildContextMenu(IMenuHost menuHost, IEnumerable<IStorageDataObject> selection, bool isContext)
        {
        }

        public void OnMenuItemClick(string itemName)
        {
        }
    }
}
```

##### IStorageContextMenu.BuildContextMenu #####

Метод вызывается перед построением контекстного меню *Pilot-Storage*.

```cpp
void IStorageContextMenu.BuildContextMenu(IMenuHost menuHost, IEnumerable<IStorageDataObject> selection);
```
где:
 - `menuHost` - интерфейс, позволяющий добавлять различные элементы меню;
 - `selection` - список выбранных элементов;


##### IStorageContextMenu.OnMenuItemClick #####

Метод вызывается при нажатии на пункт контекстного меню.

```cpp
void IStorageContextMenu.OnMenuItemClick(string itemName);
```
где:
 - `itemName` - уникальное название пункта контекстного меню.



<a name="IAutoimportHandler"/>
### Интерфейс IAutoimportHandler ###

Интерфейс позволяет реализовать обработку автоимпорта документов (печать на Pilot XPS принтер, загрузка документов в папку автоимпорта). Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IAutoimportHandler` и обязательно пометить класс, реализующий интерфейс **IAutoimportHandler** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IAutoimportHandler))]
    public class ContextMenuPlugin : IAutoimportHandler
    {
        public bool Handle(string filePath, string sourceFilePath, AutoimportSource autoimportSource)
        {
        }
    }
}
```



<a name="IPrintHandler"/>
### Интерфейс IPrintHandler ###

Интерфейс позволяет реализовать обработку печати документов с помощью принтера. Для этого необходимо реализовать интерфейс из пакета Ascon.Pilot.SDK `IPrintHandler` и обязательно пометить класс, реализующий интерфейс **IPrintHandler** атрибутом `[Export]`

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ContextMenuSample
{
    [Export(typeof(IAutoimportHandler))]
    public class ContextMenuPlugin : IAutoimportHandler
    {
        public bool Handle(IPrintedDocumentInfo printTicket)
        {
        }
    }
}
```



<a name="IObjectCardHandler"/>
### Интерфейс IObjectCardHandler ###

Интерфейс позволяет реализовать работу с атрибутами, отображаемыми в карточке объекта. Для этого необходимо реализовать интерфейс из пакета `Ascon.Pilot.SDK.IObjectCardHandler` и обязательно пометить класс, реализующий интерфейс **IObjectCardHandler** атрибутом `[Export]`.

Метод `Handle` вызывается при каждом показе карточки объекта. В случае, если карточка находится в режиме редактирования существующего объекта, свойство `EditingObject` аргумента `context` возвращает сам объект, атрибуты которого показаны в карточке; и `null`, если карточка показана для создания нового объекта.

Метод `OnValueChanged` вызывается при изменении пользователем значения какого-либо из отображаемых атрибутов в карточке объекта. Аргумент `sender` возвращает атрибут, значение которого было изменено. Аргумент `args` позволяет получить доступ к аргументам события: предыдущему и новому значению атрибута, а также контексту карточки объекта. `modifier` позволяет отреагировать на изменения и установить новое значение дополнительно одному или нескольким атрибутам. Изменения в значениях атрибутов, которые были сделаны с помощью `modifier` не приводят к вызовам метода `OnValueChanged`.

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ObjectCardHandlerSample
{
    [Export(typeof(IObjectCardHandler))]
    public class ObjectCardHandler : IObjectCardHandler
    {
        public bool Handle(IAttributeModifier modifier, ObjectCardContext context)
        {
        	var isObjectModification = context.EditiedObject != null;
        }
        
        public bool OnValueChanged(IAttribute sender, AttributeValueChangedEventArgs args, IAttributeModifier modifier)
        {
        }
    }
}
```

<a name="IAttributeModifier"/>
#### Редактирования атрибутов в карточке IAttributeModifier ####

Интерфейс позволяет установить новое значения для атрибутов элемента перед тем, как они будут показаны в карточке объекта.

##### IAttributeModifier.SetValue #####

Метод позволяет задать новое значение атрибуту.

```cpp
void IAttributeModifier.SetValue(string name, string value);
```
где:
 - `name` - уникальное наименование атрибута типа.
 - `value` - новое значение атрибута.

Метод имеет несколько перегрузок:
```cpp
void IAttributeModifier.SetValue(string name, string value);
void IAttributeModifier.SetValue(string name, long value);
void IAttributeModifier.SetValue(string name, double value);
void IAttributeModifier.SetValue(string name, DateTime value);
void IAttributeModifier.SetValue(string name, decimal value);
```

<a name="ObjectCardContext"/>
#### Контекст карточки ObjectCardContext ####

Контекст отображения карточки.

##### ObjectCardContext.DisplayAttributes #####

Поле содержит список отображаемых в карточке атрибутов. В этот список не входят сервисные атрибуты.

```cpp
IEnumerable<IAttribute> ObjectCardContext.DisplayAttributes { get; }
```

##### ObjectCardContext.AttributeValues #####

Словарь текущих значений атрибутов карточки объекта, где ключ это имя атрибута, а значение это значение атрибута.

```cpp
IDictionary<string, object> ObjectCardContext.AttributeValues { get; }
```

##### ObjectCardContext.Type #####

Поле содержит описание типа элемента для которого показана карточка.

```cpp
IType ObjectCardContext.Type { get; }
```

##### ObjectCardContext.EditiedObject #####

Текущий редактируемый объект, для которого показана карточка. Null, если карточка показана для создания нового объекта.

```cpp
IDataObject ObjectCardContext.EditiedObject { get; }
```

##### ObjectCardContext.Parent #####

Поле содержит описание родительского элемента.

```cpp
IDataObject ObjectCardContext.Parent { get; }
```

##### ObjectCardContext.IsReadOnly #####

True, если карточка в режиме только для чтения и атрибуты недоступны для изменения.

```cpp
bool ObjectCardContext.IsReadOnly { get; }
```




<a name="DataManagementInterfaces"/>
Интерфейсы управления данными
-----------------------------

Для получения доступа к интерфейсам их необходимо передать в конструктор. Можно передавать любые из перечисленных ниже интерфейсов. При этом конструктор обязательно необходимо пометить атрибутом `[ImportingConstructor]`.

```cpp
[Export(typeof(IObjectContextMenu))]
public class ModifyObjectsPlugin : IObjectContextMenu
{
     private readonly IObjectModifier _modifier;
     private readonly Repository _repository;

     [ImportingConstructor]
     public ModifyObjectsPlugin(IObjectModifier modifier, IObjectsRepository repository)
     {
         _modifier = modifier;
         _repository = repository;
     }
}
```

<a name="IObjectsRepository"/>
### Интерфейс IObjectsRepository ###

Интерфейс позволяет получить доступ к элементам, типам элементов и организационной структуры. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### IObjectsRepository.GetCurrentPerson #####
Метод позволяет получить информацию о текущем пользователе, вошедшем в систему.

```cpp
IPerson IObjectsRepository.GetCurrentPerson();
```
Возвращает объект типа `IPerson`.


##### IObjectsRepository.GetOrganisationUnit #####

Метод позволяет получить информацию об организационной единице.

```cpp
IOrganisationUnit IObjectsRepository.GetOrganisationUnit(int id);
```
где:
- `id` - идентификатор должности или подразделения.

Возвращает объект типа `IOrganisationUnit`.


##### IObjectsRepository.GetOrganisationUnits #####

Метод позволяет получить список организационных единиц.

```cpp
IEnumerable<IOrganisationUnit> IObjectsRepository.GetOrganisationUnits();
```
где:
  - `id` - идентификатор должности или подразделения.


##### IObjectsRepository.GetPeople #####

Метод позволяет получить список всех пользователей.

```cpp
IEnumerable<IPerson> IObjectsRepository.GetPeople();
```


##### IObjectsRepository.GetPerson #####

Метод позволяет получить информацию о пользователе по идентификатору.

```cpp
IPerson IObjectsRepository.GetPerson(int id);
```
где:
  - `id` - идентификатор пользователя.


##### IObjectsRepository.GetType #####

Метод позволяет получить информацию о типе элемента по идентификатору или по имени.

```cpp
IType IObjectsRepository.GetType(int id);
```
где:
  - `id` - идентификатор типа элемента.

```cpp
IType IObjectsRepository.GetType(string name);
```
где:
  - `name` - имя типа элемента.


##### IObjectsRepository.GetTypes #####

Метод позволяет получить список всех типов.

```cpp
IEnumerable<IType> IObjectsRepository.GetTypes();
```


##### IObjectsRepository.SubscribeObjects #####

Метод предназначен для получения элементов по их идентификаторам. Метод работает асинхронно. Возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных элементов. Все запрошенные элементы будут загружены в методе `OnNext`.

```cpp
IObservable<IDataObject> IObjectsRepository.SubscribeObjects(IEnumerable<Guid> ids);
```
где:
  - `ids` - список идентификаторов элементов.

Пример:
```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ObjectsSample
{
    [Export(typeof(IDataPlugin))]
    public class ObjectsSamplePlugin : IDataPlugin, IObserver<IDataObject>
    {
        private readonly IObjectsRepository _repository;
        private readonly List<IDataObject> _elements;

        [ImportingConstructor]
        public ObjectsSamplePlugin(IObjectsRepository repository)
        {
            _repository = repository;
            _elements = new List<IDataObject>();
            //Получить корневой элемент
            _repository.SubscribeObjects(new[] { SystemObjectIds.RootObjectId }).Subscribe(this);
        }

        public void OnNext(IDataObject value)
        {
        	_elements.Clear();
            if (!_elements.Exists(e => e.Id == value.Id))
                _elements.Add(value);
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }
    }
}
```
В данном примере описан сценарий получения элементов асинхронно с использованием метода `SubscribeObjects`. В данном подходе при реализации интерфейса `IObserver<IDataObject>` необходимо оставить пустой реализацию методов `OnError` и `OnCompleted` во избежании ошибок в будущем. В методе `OnNext` реализовано сохранение запрошенных элементов для дальнейшего использования.


##### IObjectsRepository.SubscribeOrganisationUnits #####

Метод предназначен для подписки на изменения организационных единиц (должности и подразделения). Метод возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных объектов. Все запрошенные объекты будут загружены в методе `OnNext`.

```cpp
IObservable<IOrganisationUnit> IObjectsRepository.SubscribeOrganisationUnits();
```
Пример:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ObjectsSample
{
    [Export(typeof(IDataPlugin))]
    public class ObjectsSamplePlugin : IDataPlugin, IObserver<IOrganisationUnit>
    {
        [ImportingConstructor]
        public ObjectsSamplePlugin(IObjectsRepository repository)
        {
            repository.SubscribeOrganisationUnits().Subscribe(this);
        }

        public void OnNext(IOrganisationUnit value)
        {
        	//TODO update organization unit
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }
    }
}
```
В данном примере описан сценарий обновления организационной единицы с использованием метода `SubscribeOrganisationUnits`. В данном подходе при реализации интерфейса `IObserver<IOrganisationUnit>` необходимо оставить пустой реализацию методов `OnError` и `OnCompleted` во избежании ошибок в будущем. В методе `OnNext` необходимо реализовать обновление организационной единицы.


##### IObjectsRepository.SubscribePeople #####

Метод предназначен для подписки на изменения информации о пользователях. Метод возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных объектов. Все запрошенные объекты будут загружены в методе `OnNext`.

```cpp
IObservable<IPerson> IObjectsRepository.SubscribePeople();
```
Пример:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ObjectsSample
{
    [Export(typeof(IDataPlugin))]
    public class ObjectsSamplePlugin : IDataPlugin, IObserver<IPerson>
    {
        [ImportingConstructor]
        public ObjectsSamplePlugin(IObjectsRepository repository)
        {
            repository.SubscribePeople().Subscribe(this);
        }

        public void OnNext(IPerson value)
        {
        	//TODO update person info
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }
    }
}
```
В данном примере описан сценарий обновления информации о пользователе с использованием метода `SubscribePeople`. В данном подходе при реализации интерфейса `IObserver<IPerson>` необходимо оставить пустой реализацию методов `OnError` и `OnCompleted` во избежании ошибок в будущем. В методе `OnNext` необходимо реализовать обновление организационной единицы.


##### IObjectsRepository.SubscribeTypes #####

Метод предназначен для подписки на изменения описания типов элементов. Метод возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных типов. Все запрошенные типы будут загружены в методе `OnNext`.

```cpp
IObservable<IType> IObjectsRepository.SubscribeTypes();
```
Пример:

```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace ObjectsSample
{
    [Export(typeof(IDataPlugin))]
    public class ObjectsSamplePlugin : IDataPlugin, IObserver<IType>
    {
        [ImportingConstructor]
        public ObjectsSamplePlugin(IObjectsRepository repository)
        {
            repository.SubscribeTypes().Subscribe(this);
        }

        public void OnNext(IType value)
        {
        	//TODO update element type info
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }
    }
}
```
В данном примере описан сценарий обновления информации о типе элемента с использованием метода `SubscribeTypes`. В данном подходе при реализации интерфейса `IObserver<IType>` необходимо оставить пустой реализацию методов `OnError` и `OnCompleted` во избежании ошибок в будущем. В методе `OnNext` необходимо реализовать обновление описания типа.

##### IObjectsRepository.SubscribeTasks #####

Метод предназначен для получения заданий по их идентификаторам. Метод работает асинхронно. Возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных элементов. Все запрошенные элементы будут загружены в методе `OnNext`.

```cpp
IObservable<ITaskObject> SubscribeTasks(IEnumerable<Guid> ids);
```
где:
  - `ids` - список идентификаторов заданий.

Пример:
```cpp
using System.ComponentModel.Composition;
using Ascon.Pilot.SDK;

namespace TaskSample
{
    [Export(typeof(IDataPlugin))]
    public class ObjectsSamplePlugin : IDataPlugin, IObserver<ITaskObject>
    {
        private readonly IObjectsRepository _repository;
        private readonly List<ITaskObject> _tasks;

        [ImportingConstructor]
        public ObjectsSamplePlugin(IObjectsRepository repository)
        {
            _repository = repository;
            _tasks = new List<IDataObject>();
            _repository.SubscribeTasks(new[] { id1, id2, id3 }).Subscribe(this);
        }

        public void OnNext(IDataObject value)
        {
        	_tasks.Clear();
            if (!_tasks.Exists(e => e.Id == value.Id))
                _tasks.Add(value);
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }
    }
}
```
В данном примере описан сценарий получения заданий асинхронно с использованием метода `SubscribeTasks`. В данном подходе при реализации интерфейса `IObserver<ITaskObject>` необходимо оставить пустой реализацию методов `OnError` и `OnCompleted` во избежании ошибок в будущем. В методе `OnNext` реализовано сохранение запрошенных элементов для дальнейшего использования.

##### IObjectsRepository.SubscribeWorkflow #####

Метод предназначен для получения рабочего процесса. Метод работает асинхронно. Возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных элементов. Все запрошенные элементы будут загружены в методе `OnNext`.

```cpp
IObservable<IWorkflowObject> SubscribeWorkflow(IEnumerable<Guid> ids);
```
где:
  - `ids` - список идентификаторов рабочих процессов.



##### IObjectsRepository.SubscribeStageObjects #####

Метод предназначен для получения этапов рабочего процесса. Метод работает асинхронно. Возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных элементов. Все запрошенные элементы будут загружены в методе `OnNext`.

```cpp
IObservable<IStageObject> SubscribeStageObjects(IEnumerable<Guid> ids);
```
где:
  - `ids` - список идентификаторов этапов.



##### IObjectsRepository.SubscribeTaskMessages #####

Метод предназначен для получения переписки по заданию. Метод работает асинхронно. Возвращает стандартный интерфейс [IObservable](https://msdn.microsoft.com/en-us/library/dd990377.aspx), который позволяет подписаться на изменения запрошенных элементов. Все запрошенные элементы будут загружены в методе `OnNext`.

```cpp
IObservable<ITaskMessage> SubscribeTaskMessages(IEnumerable<Guid> ids);
```
где:
  - `ids` - список идентификаторов сообщений.


<a name="IObjectModifier"/>
### Интерфейс IObjectModifier ###

Интерфейс предназначен для создания, изменения и удаление элементов. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.


##### IObjectModifier.Create #####

Метод позволяет создать новый элемент заданного типа.

```cpp
IObjectBuilder IObjectModifier.Create(IDataObject parent, IType type);
```
где:
 - `parent` - родительский элемент, в котором создается новый элемент.
 - `type` - тип создаваемого элемента.

Метод возвращает объект типа `IObjectBuilder`, который позволяет заполнить элемент атрибутами и задать дополнительные свойства. Подробнее смотри интерфейс IObjectBuilder


##### IObjectModifier.Edit #####

Метод позволяет отредактировать заданный элемент.

```cpp
IObjectBuilder IObjectModifier.Edit(IDataObject object);
```
где:
 - `object` - редактируемые элемент.

Метод возвращает объект типа `IObjectBuilder`, который позволяет заполнить элемент атрибутами и задать дополнительные свойства. Подробнее смотри интерфейс IObjectBuilder


##### IObjectModifier.Move #####

Метод позволяет переместить элемент из текущего родительского в другой.

```cpp
void IObjectModifier.Move(IDataObject object, IDataObject newParent);
```
где:
 - `object` - текущий элемент, который надо переместить;
 - `newParent` - новый родительский элемент.


##### IObjectModifier.Delete #####

Метод позволяет удалить элемент в корзину.

```cpp
void IObjectModifier.Delete(IDataObject object);
```
где:
 - `object` - элемент, который надо удалить.

##### IObjectModifier.DeleteById #####

Метод позволяет удалить элемент в корзину.

```cpp
void IObjectModifier.DeleteById(Guid objectId);
```
где:
 - `objectId` - идентификатор элемента, который надо удалить.
 
##### IObjectModifier.ChangeState #####

Метод позволяет изменить статус элемента.

```cpp
void IObjectModifier.ChangeState(IDataObject @object, ObjectState state);
```
где:
 - `object` - элемент, статус которого нужно изменить
 - `state` - новый статус элемента.

##### IObjectModifier.CreateLink #####

Метод позволяет связать два элемента выбранной связью.

```cpp
void IObjectModifier.CreateLink(IRelation relation1, IRelation relation2);
```
где:
 - `relation1` - описание связи для первого объекта
 - `relation2` - описание связи для второго объекта

Пример:
```cpp
using Ascon.Pilot.SDK;

class ObjectsSample
{
    private IObjectModifier _modifier;

    {...}

    private void CreateSourceFileLink(ObjectsViewContext context)
    {
        var object1 = context.SelectedObjects.First();
        var object2 = context.SelectedObjects.Last();

        //Create relations
        var relationId = Guid.NewGuid();
        var relationName = "RelationName";
        var relationType = ObjectRelationType.Custom;

        var relation1 = new Relation
        {
            Id = relationId,
            Type = relationType,
            Name = relationName,
            TargetId = object2.Id
        };

        var relation2 = new Relation
        {
            Id = relationId,
            Type = relationType,
            Name = relationName,
            TargetId = object1.Id
        };

        _modifier.CreateLink(relation1, relation2);
        _modifier.Apply();
    }

    {...}
}
```

##### IObjectModifier.RemoveLink #####

Метод позволяет удалить связь между объектами.

```cpp
void IObjectModifier.RemoveLink(IDataObject obj, IRelation relation);
```
где:
 - `obj` - объект, из которого будет удалена связь
 - `relation` - удаляемая связь

Пример:
```cpp
using Ascon.Pilot.SDK;

class ObjectsSample
{
    private IObjectModifier _modifier;

    {...}

    private void RemoveSourceFileLink(ObjectsViewContext context)
    {
		var selected = context.SelectedObjects.First();

		foreach (var relation in selected.Relations.Where(x => x.Type == ObjectRelationType.SourceFiles))
		{
			_modifier.RemoveLink(selected, relation);
		}
        _modifier.Apply();
    }

    {...}
}
```

##### IObjectModifier.Apply #####

Применить все изменения сделанные над элементами. Без вызова этого метода изменения не будут применены.

```cpp
void IObjectModifier.Apply();
```


<a name="IObjectBuilder"/>
### Интерфейс IObjectBuilder ###

Интерфейс обеспечивает возможность наполнения или редактирования элементов атрибутами и дополнительными свойствами.


##### IObjectBuilder.SetAttribute #####

Метод задает или изменяет атрибут с заданным именем.

```cpp
IObjectBuilder SetAttribute(string name, string value);
```
где:
 - `name` - имя атрибута.
 - `value` - новое значение атрибута.

Метод имеет несколько перегрузок:
`IObjectBuilder SetAttribute(string name, string value);` - для установки строкового значения атрибута.
`IObjectBuilder SetAttribute(string name, int value);` - для установки целочисленного значения атрибута.
`IObjectBuilder SetAttribute(string name, DateTime value);` - для установки значения атрибута времени.
`IObjectBuilder SetAttribute(string name, double value);` - для установки цифрового значения атрибута с плавающей точкой.


#### IObjectBuilder.RemoveAttribute #####

Метод удаляет атрибут с заданным именем.

```cpp
IObjectBuilder RemoveAttribute(string name);
```
где:
 - `name` - имя атрибута.


##### IObjectBuilder.DataObject #####

Поле возвращает пустую болванку редактируемого элемента с заполненным идентификатором элемента и идентификатором родительского элемента.

```cpp
IDataObject IObjectBuilder.DataObject { get; }
```


##### IObjectBuilder.AddFile #####

Метод добавляет файл к заданному элементу.
```cpp
void IObjectBuilder.AddFile(string path);
```
где:
 - `path` - путь до файла, который надо добавить.

Этот метод имеет перегрузку:
```cpp
void IObjectBuilder.AddFile(string name, Stream stream, DateTime creationTime, DateTime lastAccessTime, DateTime lastWriteTime);
```
где:
 - `name` - имя добавляемого файла.
 - `stream` - поток добавляемого файла.
 - `creationTime` - дата создания файла.
 - `lastAccessTime` - дата последнего доступа к файлу.
 - `lastWriteTime` - дата последнего доступа на запись в файл.

##### IObjectBuilder.AddOrReplaceFile #####
Метод добавляет или заменяет файл к заданному элементу.
```cpp
void IObjectBuilder.AddOrReplaceFile(string name, Stream stream, IFile file, DateTime creationTime, DateTime lastAccessTime, DateTime lastWriteTime);
```
где:
 - `name` - имя файла.
 - `stream` - поток файла.
 - `file` - описание файла.
 - `creationTime` - дата создания файла.
 - `lastAccessTime` - дата последнего чтения из файла.
 - `lastWriteTime` - дата последнего изменения файла.



##### IObjectBuilder.SetAccessRights #####

Метод добавляет директивные права на элемент.
```cpp
IObjectBuilder SetAccessRights(int positionId, AccessLevel level, DateTime validThrough, bool isInherited);
```
где:
 - `positionId` - идентификатор должности пользователя.
 - `level` - уровень доступа к элементу.
 - `validThrough` - срок действия права.
 - `isInherited` - наследовать это право или нет.



##### IObjectBuilder.RemoveAccessRights #####

Метод удаляет директивные права на элемент.
```cpp
IObjectBuilder RemoveAccessRights(int positionId);
```
где:
 - `positionId` - идентификатор должности пользователя.


##### IObjectBuilder.MakeSecret #####
Метод позволяет сделать элемент скрытым.
```cpp
IObjectBuilder MakeSecret();
```

##### IObjectBuilder.MakePublic #####
Метод позволяет сделать элемент общедоступным.
```cpp
IObjectBuilder MakePublic();
```

##### IObjectBuilder.CreateFileSnapshot #####
Метод позволяет создать снимок файлов и указать причину. Текущие файлы будут перемещены в историю.
```cpp
IObjectBuilder CreateFileSnapshot(string reason);
```
где:
 - `reason` - причина создания снимка файлов.

##### IObjectBuilder.MakeSnapshotActual #####
Метод позволяет сделать выбранный снимок файлов текущим.
```cpp
IObjectBuilder MakeSnapshotActual(string reason, IFilesSnapshot snapshot);
```
где:
 - `reason` - причина создания снимка файлов.
 - `snapshot` - снимок, который надо сделать текущим (актуальным).


##### IObjectBuilder.AddSubscriber #####
Метод позволяет подписаться на изменения элемента.
```cpp
IObjectBuilder AddSubscriber(int personId);
```
где:
 - `personId` - идентификатор пользователя.

##### IObjectBuilder.RemoveSubscriber #####
Метод позволяет отписаться от изменений элемента.
```cpp
IObjectBuilder RemoveSubscriber(int personId);
```
где:
 - `personId` - идентификатор пользователя.

##### IObjectBuilder.SetIsDeleted #####
Метод позволяет задать элементу состояние удалено безвозвратно.
```cpp
IObjectBuilder SetIsDeleted(bool isDeleted);
```
где:
 - `isDeleted` - флаг указывающий удалить объект или нет.

##### IObjectBuilder.SetIsInRecycleBin #####
Метод позволяет задать элементу состояние удалено корзину.
```cpp
IObjectBuilder SetIsInRecycleBin(bool isInRecycleBin);
```
где:
 - `isInRecycleBin` - флаг указывающий удалить объект в корзину или нет.

##### IObjectBuilder.SetParent #####
Метод позволяет задать элементу нового родителя.
```cpp
IObjectBuilder SetParent(Guid parentId);
```
где:
 - `parentId` - идентификатор нового родителя.

##### IObjectBuilder.SetType #####
Метод позволяет задать элементу тип.
```cpp
IObjectBuilder SetType(IType type);
```
где:
 - `type` - тип элемента.

##### IObjectBuilder.SetCreator #####
Метод позволяет задать создателя элемента.
```cpp
IObjectBuilder SetCreator(int creatorId);
```
где:
 - `creatorId` - идентификатор пользователя (не должности).

##### IObjectBuilder.Lock #####
Метод позволяет заблокировать объект для изменений текущим пользователем.
```cpp
IObjectBuilder Lock();
```

##### IObjectBuilder.Unlock #####
Метод позволяет разблокировать объект.
```cpp
IObjectBuilder Unlock();
```


<a name="ITaskModifier"/>
### Интерфейс ITaskModifier ###

Интерфейс предназначен для создания и изменения заданий. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### ITaskModifier.NewWorkflow #####
Метод создает новый рабочий процесс для заданий. Один рабочий процесс может содержать множество этапов.
```cpp
IWorkflowChangeBuilder NewWorkflow();
```
Метод возвращает объект типа [IWorkflowChangeBuilder](#IWorkflowChangeBuilder), который позволяет заполнить элемент атрибутами и задать дополнительные свойства.

##### ITaskModifier.NewStage #####
Метод создает новый этап в рабочем процессе. Один этап может содержать множество заданий.
```cpp
IStageChangeBuilder NewStage(IWorkflowObject workflow);
```
где:
 - `workflow` - рабочий процесс, в котором надо создать этап.
Метод возвращает объект типа [IStageChangeBuilder](#IStageChangeBuilder), который позволяет заполнить элемент атрибутами и задать дополнительные свойства.

##### ITaskModifier.NewTask #####
Метод позволяет создать новое задание или несколько заданий для разных исполнителей.
```cpp
IEnumerable<ITaskBuilder> NewTask(string title, string description, IEnumerable<int> executors, DateTime deadLine);
```
где:
 - `title` - тема задания.
 - `description` - текст задания.
 - `executors` - список должностей исполнителей задания.
 - `deadLine` - срок исполнения задания.

Метод возвращает список объектов `IEnumerable<ITaskBuilder>`, который позволяет заполнить элементы атрибутами и задать дополнительные свойства для каждого задания. Подробнее смотри [ITaskBuilder](#ITaskBuilder).

```cpp
IEnumerable<ITaskBuilder> NewTask(string title, string description, IEnumerable<int> executors, DateTime deadLine, IEnumerable<Guid> attachments);
```
где:
 - `title` - тема задания.
 - `description` - текст задания.
 - `executors` - список должностей исполнителей задания.
 - `deadLine` - срок исполнения задания.
 - `attachments` - список идентификаторов элементов, которые можно добавить в задание.

```cpp
ITaskBuilder NewTask(IWorkflowObject workflow, IStageObject stage, int executorPosition);
```
где:
 - `workflow` - родительский объект типа workflow.
 - `stage` - этап задания.
 - `executorPosition` - должность исполнителя задания.


##### ITaskModifier.NewAgreementTask #####
Метод позволяет создать новое задание на согласование.
```cpp
ITaskBuilder NewAgreementTask(IWorkflowObject workflow, IStageObject stage, int executorPosition);
```
где:
 - `workflow` - рабочий процесс заданий.
 - `stage` - этап.
 - `executorPosition` - должность исполнителя задания.


##### ITaskModifier.EditTask #####
Метод позволяет отредактировать задание.
```cpp
ITaskBuilder EditTask(ITaskObject task);
```
где:
 - `task` - редактируемое задание.

Метод возвращает объект типа [ITaskBuilder](#ITaskBuilder), который позволяет заполнить элемент атрибутами и задать дополнительные свойства. При вызове этого метода создается версия задания и сохраняется в истории. Изменения вступают в силу только после вызова метода `Apply`

##### ITaskModifier.EditTaskById #####
Метод изменения задания по идентификатору объекта задания.
```cpp
ITaskBuilder EditTaskById(Guid taskId);
```
где:
 - `taskId` - идентификатор редактируемого задания.

Метод возвращает объект типа [ITaskBuilder](#ITaskBuilder), который позволяет заполнить элемент атрибутами и задать дополнительные свойства. При вызове этого метода создается версия задания и сохраняется в истории. Изменения вступают в силу только после вызова метода `Apply`

##### ITaskModifier.EditWorkflow #####
Метод изменения рабочего процесса заданий.
```cpp
IWorkflowChangeBuilder EditWorkflow(IWorkflowObject workflow);
```
где:
 - `workflow` - редактируемый рабочий процесс.
Метод возвращает объект типа [IWorkflowChangeBuilder](#IWorkflowChangeBuilder), который позволяет заполнить элемент атрибутами и задать дополнительные свойства.


##### ITaskModifier.EditStage #####
Метод изменения этапа заданий.
```cpp
IStageChangeBuilder EditStage(IStageObject stage);
```
где:
 - `stage` - редактируемый этап.
Метод возвращает объект типа [IStageChangeBuilder](#IStageChangeBuilder), который позволяет заполнить элемент атрибутами и задать дополнительные свойства.

##### ITaskModifier.Apply #####

Применить все изменения сделанные над заданием. Без вызова этого метода изменения не будут применены.

```cpp
void ITaskModifier.Apply();
```

<a name="IWorkflowChangeBuilder"/>
### Интерфейс IWorkflowChangeBuilder ###
Интерфейс обеспечивает возможность наполнения или редактирования рабочего процесса заданий атрибутами и дополнительными свойствами.

##### IWorkflowChangeBuilder.SetTitle #####
Метод задает или изменяет заголовок рабочего процесса. Этот заголовок распространится на все этапы и задания рабочего процесса.
```cpp
IWorkflowChangeBuilder SetTitle(string title);
```
где:
 - `title` - заголовок (тема) рабочего процесса.

##### IWorkflowChangeBuilder.SetDescription #####
Метод задает или изменяет описание рабочего процесса. Это описание распространится на все этапы и задания рабочего процесса.
```cpp
IWorkflowChangeBuilder SetDescription(string description);
```
где:
 - `description` - описание рабочего процесса.

##### IWorkflowChangeBuilder.SetInitiatorAttachments #####
Метод задает или изменяет вложения инициатора рабочего процесса. Все вложения так же будут доступны во всех этапах и заданиях рабочего процесса.
```cpp
IWorkflowChangeBuilder SetInitiatorAttachments(IEnumerable<Guid> attachments);
```
где:
 - `attachments` - список идентификаторов элементов.

##### IWorkflowChangeBuilder.WorkflowObject #####
Поле содержит описание объекта рабочего процесса. Подробнее см. [IWorkflowObject](#IWorkflowObject)
```cpp
IWorkflowObject WorkflowObject { get; }
```

<a name="IStageChangeBuilder"/>
### Интерфейс IStageChangeBuilder ###
Интерфейс обеспечивает возможность наполнения или редактирования этапа заданий атрибутами и дополнительными свойствами.

##### IStageChangeBuilder.SetStageOrder #####
Метод задает или изменяет порядок выполнения этапа.
```cpp
IStageChangeBuilder SetStageOrder(int order);
```
где:
 - `order` - порядковый номер этапа.

##### IStageChangeBuilder.StageObject #####
Поле содержит описание объекта тапа. Подробнее см. [IStageObject](#IStageObject)
```cpp
IStageObject StageObject { get; }
```


<a name="ITaskBuilder"/>
### Интерфейс ITaskBuilder ###
Интерфейс обеспечивает возможность наполнения или редактирования заданий атрибутами и дополнительными свойствами.

##### ITaskBuilder.SetAttribute #####
Метод задает или изменяет атрибут с заданным именем.
```cpp
ITaskBuilder SetAttribute(string name, string value);
```
где:
 - `name` - имя атрибута.
 - `value` - новое значение атрибута.

Метод имеет несколько перегрузок:
`ITaskBuilder SetAttribute(string name, string value);` - для установки строкового значения атрибута.
`ITaskBuilder SetAttribute(string name, int value);` - для установки целочисленного значения атрибута.
`ITaskBuilder SetAttribute(string name, DateTime value);` - для установки значения атрибута времени.
`ITaskBuilder SetAttribute(string name, double value);` - для установки цифрового значения атрибута с плавающей точкой.

#### ITaskBuilder.RemoveAttribute #####
Метод удаляет атрибут с заданным именем.
```cpp
ITaskBuilder RemoveAttribute(string name);
```
где:
 - `name` - имя атрибута.

#### ITaskBuilder.SetTitle #####
Метод задает новую тему задания.
```cpp
ITaskBuilder SetTitle(string title);
```
где:
 - `title` - новая тема задания.

#### ITaskBuilder.SetDescription #####
Метод задает новый текст задания.
```cpp
ITaskBuilder SetDescription(string description);
```
где:
 - `description` - новый текст задания.

#### ITaskBuilder.SetState #####
Метод изменяет состояние задания.
```cpp
ITaskBuilder SetState(State state);
```
где:
 - `state` - новое состояние задание.

#### ITaskBuilder.SetExecutor #####
Метод изменяет исполнителя задания.
```cpp
ITaskBuilder SetExecutor(IPosition position);
```
где:
 - `position` - новая должность исполнителя задания.

#### ITaskBuilder.SetDeadline #####
Метод изменяет срок окончания работ по заданию.
```cpp
ITaskBuilder SetDeadline(DateTime deadlineDate);
```
где:
 - `deadlineDate` - новый срок окончания работ по заданию.

#### ITaskBuilder.SetRelativeDeadline #####
Задает относительный срок окончания работ по заданию. Применим к заданиям второго и последующих этапов. Метод вычисляет количество дней между сегодняшним днем и датой, переданной в качестве аргумента. Это значение будет использованно для вычисления срока окончания работ по заданию нового этапа в момент завершения предыдущего этапа согласования: дата, когда завершился предыдущий этап, плюс определенное ранее количество дней. Если задать относительный срок окончания работ для задания первого этапа, такое задание не получит срока вообще, потому что нет предыдущего этапа. Отменить установку относительного срока задания можно передав значение DateTime.MaxValue в качестве аргумента.
```cpp
ITaskBuilder SetRelativeDeadline(DateTime relativeDeadline);
```
где:
 - `deadlineDate` - новая дата, которая используется для вычисления количества дней относительно сегодняшней даты, которое будет предоставлено для выполнения задания, после окончания предыдущего этапа.

#### ITaskBuilder.SetInitiatorAttachments #####
Метод изменяет набор вложенных инициатором элементов в задание.
```cpp
ITaskBuilder SetInitiatorAttachments(IEnumerable<Guid> attachments);
```
где:
 - `attachments` - новый список идентификаторов элементов.

#### ITaskBuilder.SetExecutorAttachments #####
Метод изменяет набор вложенных исполнителем элементов в задание.
```cpp
ITaskBuilder SetExecutorAttachments(IEnumerable<Guid> attachments);
```
где:
 - `attachments` - новый список идентификаторов элементов.

#### ITaskBuilder.SetExecutorRole #####
Метод позволяет задать роль исполнителя задания при согласовании документов.
```cpp
ITaskBuilder SetExecutorRole(string role);
```
где:
 - `role` - новая роль.

#### ITaskBuilder.SetValidationPhase #####
Метод задает значение, которое указывает будет ли задание иметь состояние "на проверке".
```cpp
ITaskBuilder SetValidationPhase(bool withValidation);
```
где:
 - `withValidation` - флаг, указывающий будет ли задание иметь фазу проверки.

#### ITaskBuilder.SetAuditors #####
Метод позволяет задать аудиторов задания.
```cpp
ITaskBuilder SetAuditors(IEnumerable<int> positionIds);
```
где:
 - `positionIds` - список идентификаторов должностей аудиторов.

#### ITaskBuilder.SetTags #####
Метод позволяет задать тэги.
```cpp
ITaskBuilder SetTags(IEnumerable<string> tags);
```
где:
 - `tags` - список тэгов.

#### ITaskBuilder.SetKind #####
Метод позволяет задать тип задания.
```cpp
ITaskBuilder SetKind(TaskKind kind);
```
где:
 - `kind` - тип задания.

#### ITaskBuilder.TaskObject #####
Текущий объект задания.
```cpp
ITaskObject TaskObject { get; }
```


<a name="IFileProvider"/>
### Интерфейс IFileProvider ###
Интерфейс обеспечивает работу с телами файлов. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### IFileProvider.Exists #####
Метод показывает существует-ли тело файла с заданным идентификатором файла.
```cpp
bool IFileProvider.Exists(Guid fileId);
```
где:
 - `fileId` - идентификатор файла.

##### IFileProvider.IsFull #####
Метод показывает полностью-ли загружен файл с сервера. 
```cpp
bool IFileProvider.IsFull(Guid fileId);
```
где:
 - `fileId` - идентификатор файла.

##### IFileProvider.OpenRead #####
Метод открывает заданный файл на чтение. Загружает тело файла, если файл не был загружен с сервера.
```cpp
Stream IFileProvider.OpenRead(IFile file);
```
где:
 - `file` - открываемый файл.

<a name="DataProccessingInterfaces"/>
Интерфейсы работы с данными
-----------------------------

<a name="IXpsDigitalSigner"/>
### Интерфейс IXpsDigitalSigner ###

Интерфейс позволяет подписывать запросы на подпись в XPS документах. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### IXpsDigitalSigner.SignDocumentWithCertificate #####
Метод подписывает документ выбранным сертификатом.
```cpp
void IFileProvider.SignDocumentWithCertificate(Guid documentId, IEnumerable<int> positionIds, X509Certificate2 certificate);
```
где:
 - `documentId` - идентификатор документа.
 - `positionIds` - список идентификаторов пользователей, от имени которого необходимо подписать документ.
 - `certificate` - цифровой сертификат.

<a name="IXpsRender"/>
### Интерфейс IXpsRender ###
Интерфейс позволяет рендерить XPS документы в растровое изображение. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### IXpsRender.RenderXpsToBitmap #####

Метод возвращает список отрендеренных страниц в формате Stream.

```cpp
IEnumerable<Stream> IFileProvider.RenderXpsToBitmap(Stream xpsStream, double quality = 1.0);
```
где:
 - `xpsStream` - поток XPS документы.
 - `quality` - качество, в котором будет рэндериться (от 0 до 6).

<a name="IXpsViewer"/>
### Интерфейс IXpsViewer ###
Интерфейс позволяет получить номер текущей страницы просмотрщика документов и произвести увеличение/уменьшение масштаба на элементе графического слоя или аннотации. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### IXpsViewer.CurrentPageNumber #####

Свойство возвращает номер открытой в просмотрщике страницы (от 0 до N-1)

 ##### IXpsViewer.ZoomToElement #####

Метод производит увеличение/уменьшение масштаба на элементе графического слоя или аннотации по его идентификатору

```cpp
void ZoomToElement(Guid id, double scale = 1);
```
где:
 - `id` - Guid графического элемента или аннотации.
 - `scale` - изменение масштаба (от 0 до 1). 1 - максимальный масштаб

<a name="IXpsMerger"/>
### Интерфейс IXpsMerger ###
Интерфейс позволяет сохранять скомпонованный XPS документ по указанному пути. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.

##### IXpsMerger.SaveMergedXps #####

Метод сохраняет скомпонованный документ по пути и производит подписку на событие завершения сохранения

```cpp
void SaveMergedXps(string filepath, IXpsMergerSaveListener saveListener);
```
где:
 - `filepath` - путь для сохранения.
 - `saveListener` - слушатель события завершения сохранения документа

<a name="IXpsMergerSaveListener"/>
### Интерфейс IXpsMergerSaveListener ###
Интерфейс позволяет обработать завершение сохранения скомпонованного XPS документа. Для использования необходимо создать класс, унаследованный от абстрактного XpsMergerSaveListener, и передать его аргументом в метод IXpsMerger.SaveMergedXps

##### IXpsMergerSaveListener.OnSaveCompleted #####

Метод вызывается при завершении сохранения документа

```cpp
void OnSaveCompleted(string filepath);
```
где:
 - `filepath` - путь до сохраненного документа.

<a name="IGraphicLayerElement"/>
### Интерфейс IGraphicLayerElement ###

Интерфейс реализует объект графического слоя на XPS документах.

##### IGraphicLayerElement.ElementId #####

Идентификатор графического элемента.

##### IGraphicLayerElement.ContentId #####

Идентификатор содержимого графического элемента.

##### IGraphicLayerElement.OffsetY #####

Отступ по вертикали.

##### IGraphicLayerElement.OffsetX #####

Отступ по горизонтали.

##### IGraphicLayerElement.Scale #####

Масштаб.

##### IGraphicLayerElement.Angle #####

Угол поворота.

##### IGraphicLayerElement.VerticalAlignment #####

Вертикальное выравнивание.

##### IGraphicLayerElement.HorizontalAlignment #####

Горизонтальное выравнивание.

##### IGraphicLayerElement.ContentType #####

Тип содержимого графического элемента.

<a name="GraphicLayerElementConstants"/>
### Класс GraphicLayerElementConstants ###

Класс с константами для работы с графическим слоем

##### GraphicLayerElementConstants.GRAPHIC_LAYER_ELEMENT #####

Константа для обозначения графического элемента.

##### GraphicLayerElementConstants.GRAPHIC_LAYER_ELEMENT_CONTENT #####

Константа для обозначения содержимого графического элемента.

##### GraphicLayerElementConstants.XAML #####

Константа для обозначения содержимого XAML.

##### GraphicLayerElementConstants.BITMAP #####

Константа для обозначения содержимого растровой графики.

<a name="InterfacesToTheClient"/>
Интерфейсы взаимодействия с клиентом Pilot-ICE/ECM
--------------------------------------------------

Для получения доступа к интерфейсам их необходимо передать в конструктор. Можно передавать любые из перечисленных ниже интерфейсов. При этом конструктор обязательно необходимо пометить атрибутом `[ImportingConstructor]`.

```cpp
[Export(typeof(IObjectContextMenu))]
public class ModifyObjectsPlugin : IObjectContextMenu
{
     private readonly ITabServiceProvider _tabServiceProvider;

     [ImportingConstructor]
     public ModifyObjectsPlugin(ITabServiceProvider tabServiceProvider)
     {
         _tabServiceProvider = tabServiceProvider;
     }
}
```


<a name="ITabServiceProvider"/>
### Интерфейс ITabServiceProvider ###

Интерфейс обеспечивает работу с вкладками главного окна клиента Pilot-ICE/ECM. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.


##### ITabServiceProvider.GetTabPageTitles #####

Метод возвращает список заголовков открытых вкладок.

```cpp
IEnumerable<string> ITabServiceProvider.GetTabPageTitles();
```


##### ITabServiceProvider.GetActiveTabPageTitle #####

Метод возвращает заголовок активной вкладки.

```cpp
string ITabServiceProvider.GetActiveTabPageTitle();
```


##### ITabServiceProvider.ActivateTabPage #####

Вызов метода активирует вкладку с заданным заголовком.

```cpp
void ITabServiceProvider.ActivateTabPage(string title);
```
где:
 - `title` - заголовок вкладки для активации.


##### ITabServiceProvider.OpenNewTabPage #####

Метод открывает вкладку *Новая вкладка* в текущей панели вкладок или в новой.

```cpp
void ITabServiceProvider.OpenNewTabPage(bool isInOtherPane = false);
```
где:
 - `isInOtherPane` - флаг, указывающий где открыть новую вкладку:
 	+ `true` - открыть вкладку в новой панели вкладок;
 	+ `false` - открыть вкладку в текущей панели вкладок. Значение по умолчанию.


##### ITabServiceProvider.OpenTabPage #####

Метод открывает новую вкладку с заданным контентом в текущей панели вкладок или в новой.

```cpp
void ITabServiceProvider.OpenTabPage(string title, FrameworkElement view, bool isInOtherPane = false);
```
где:
 - `title` - локализованный заголовок открываемой вкладки;
 - `view` - контент, который будет отображаться в открываемой вкладке;
 - `isInOtherPane` - флаг, указывающий где открыть новую вкладку:
 	+ `true` - открыть вкладку в новой панели вкладок.
 	+ `false` - открыть вкладку в текущей панели вкладок. Значение по умолчанию.


##### ITabServiceProvider.CloseTabPage #####

Метод закрывает первую открытую вкладку с заданным заголовком.

```cpp
void ITabServiceProvider.CloseTabPage(string title);
```
где:
 - `title` - локализованный заголовок закрываемой вкладки.


##### ITabServiceProvider.UpdateTabPageContent #####

Метод обновляет контент в открытой вкладке с заданным заголовком.

```cpp
void ITabServiceProvider.UpdateTabPageContent(string oldTitle, string newTitle, FrameworkElement view);
```
где:
 - `oldTitle` - локализованный заголовок вкладки, в которой меняется контент;
 - `newTitle` - новый локализованный заголовок вкладки;
 - `view` - новый контент, который будет отображаться во вкладке.



<a name="IPilotDialogService"/>
### Интерфейс IPilotDialogService ###

Интерфейс обеспечивает работу со встроенными диалоговыми окнами клиента Pilot-ICE/ECM. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.


##### IPilotDialogService.ShowSharingSettingsDialog #####

Метод показывает диалог управления общим доступом к элементам.

```cpp
void IPilotDialogService.ShowSharingSettingsDialog(IEnumerable<Guid> objectIds);
```
где:
 - `objectIds` - список выбранных идентификаторов элементов.


##### IPilotDialogService.ShowPositionSelectorDialog #####

Метод показывает диалог выбора пользователей и должностей.

```cpp
IEnumerable<IOrganisationUnit> IPilotDialogService.ShowPositionSelectorDialog();
```


##### IPilotDialogService.ShowDocumentsSelectorDialog #####

Метод показывает диалог выбора элементов из обозревателя проектов.

```cpp
IEnumerable<IDataObject> IPilotDialogService.ShowDocumentsSelectorDialog();
```
Метод возвращает список выбранных элементов.


##### IPilotDialogService.ShowReferenceBookDialog #####

Метод показывает диалог выбора элементов из справочика

```cpp
IEnumerable<IDataObject> IPilotDialogService.ShowReferenceBookDialog(string referenceBookConfiguration, string serializationId = null, IPilotDialogOptions options = null);
```
где:
 - `referenceBookConfiguration` - строка конфигурации справочника (может быть получена из дополнительных параметров соответсвующего атрибута);
 - `serializationId` - идентификатор, использующийся для сохранения и восстановления состояния диалога (список развернутых узлов дерева);
 - `options` - дополнительные параметры показа диалога справочика;
```
Метод возвращает список выбранных элементов.



<a name="IEventAggregator"/>
### Интерфейс IEventAggregator ###

Интерфейс обеспечивает возможность подписываться на разные события возникающие в клиенте Pilot-ICE/ECM. Получить интерфейс можно через конструктор помеченный атрибутом `[ImportingConstructor]`.


##### IEventAggregator.Subscribe #####

Метод подписывает заданного подписчика на события.

```cpp
void IEventAggregator.Subscribe(object subscriber);
```
где:
 - `subscriber` - подписчик. Должен реализовывать один или несколько интерфейсов событий

Пример:
```cpp
using Ascon.Pilot.SDK;

namespace EventSample
{
	class EventSubscriber : IHandle<UnloadedEventArgs>, IHandle<LoadedEventArgs>
    {
        private readonly IEventAggregator _eventAggregator;

        public EventSubscriber(IEventAggregator eventAggregator)
        {
            _eventAggregator = eventAggregator;
            _eventAggregator.Subscribe(this);
        }

        public void Handle(UnloadedEventArgs message)
        {
            //TODO do something when unloading extension
        }

        public void Handle(LoadedEventArgs message)
        {
            //TODO do something when loading extension
        }
    }
}
```
В примере показано как можно подписаться на события загрузки и выгрузки расширения.


##### IEventAggregator.Unsubscribe #####

Метод отписывается от  ранее подписанных событий;

```cpp
void IEventAggregator.Unsubscribe(object subscriber);
```
где:
 - `subscriber` - подписчик. Должен реализовывать один или несколько интерфейсов событий.

<a name="IConnectionStateProvider"/>
### Интерфейс IConnectionStateProvider ###

Интерфейс для получения информации о состоянии подключения к Pilot-Server


##### IConnectionStateProvider.IsOnline #####

Метод, определяющий, установлено ли соединение клиента с Pilot-Server

```cpp
bool IsOnline();
```

##### IConnectionStateProvider.Subscribe #####

Подписка на изменение состояния подключения клиента с Pilot-Server

```cpp
IObservable<bool> Subscribe();
```

<a name="AttributeFormatInterfaces"/>
Работа с конфигурацией атрибутов: форматирование, нумераторы, справочники
--------------------------------------------------

Каждый из пользовательских атрибутов может быть дополнен конфигурацией. Конфигурация представляет собой текстовое поле, с помощью которого могут быть заданы специальные настройки атрибута:
- способ форматирования атрибутов объекта при отображении;
- описание нумератора;
- описание справочника;

Для работы с конфигурацией атрибута можно использовать интерфейс IAttributeFormatParser, который предоставляет методы для десериализации настроек, хранящихся в строке конфигурации атрибута. Более полную информацию о форматировании атрибутов, нумераторах и справочниках смотри в справке к Pilot-ICE. 

Пример *Ascon.Pilot.SDK.ObjectsSample* использует этот интерфейс для построения карточки объекта, включающей в себя отображение атрибутов, которым назначен нумератор, и атрибутов типа справочник; а так же для построения отоброжаемого имени объектов.

<a name="IAttributeFormatParser"/>
### Интерфейс IAttributeFormatParser###

Позволяет работать с конфигурайией атрибутов: получать формат отображения атрибутов, а так же десериализованное описание нумераторов и справочников.

Для получения доступа к интерфейсу поиска его необходимо передать в конструктор. При этом конструктор обязательно необходимо пометить атрибутом [ImportingConstructor].

<a name="AttributesFormat"/>
##### string AttributesFormat(string format, Dictionary<string, object> attributes) #####

Осуществляет форматирование значениий атрибутов объекта в соответсвии с указанным форматом. Например, используется для получения отображаемого имени объекта. Строка форматирования может быть получена с помощью метода GetAttributeFormat, который описан ниже.

```cpp
string objAttributesDisplayString = _parser.AttributesFormat(format, objectAttributes);
```
где:
 - `objAttributesDisplayString ` - значение, полученное при применении приведенной строки форматирования к текущему состоянию объекта;
 - `_parser ` - экземляр объекта, реализующего интерфейс IAttributeFormatParser;
 - `format ` - приведенная строка форматирования атрибутов;
 - `objectAttributes ` - текущее состояние объекта: коллекция пар <ИмяАтрибута, ЗначениеАтрибута>;


<a name="GetAttributeFormat"/>
##### string GetAttributeFormat(string configuration) #####

Возвращает строку форматирования атрибута на основе конфигурации атрибута.

```cpp
string format = _parser.GetAttributeFormat(configuration);
```
где:
 - `format ` - строка форматирования атрибутов объекта;
 - `_parser ` - экземляр объекта, реализующего интерфейс IAttributeFormatParser;
 - `configuration ` - строка конфигурации атрибута;


<a name="GetAttributeFormatCulture"/>
##### string GetAttributeFormatCulture(string configuration) #####

Возвращает имя культуры, которая должна использоваться, при форматировании значения атрибута.

```cpp
string culture = _parser.GetAttributeFormatCulture(configuration);
```
где:
 - `culture ` - строка культуры форматирования арибутов объекта;
 - `_parser ` - экземляр объекта, реализующего интерфейс IAttributeFormatParser;
 - `configuration ` - строка конфигурации атрибута;


<a name="TryParseNumeratorDeclaration"/>
##### bool TryParseNumeratorDeclaration(string declaration, out IEnumerable<INumeratorInfo> numeratorInfo) #####

Метод для получения десериализованного описания нумераторов из строки конфигурации атрибута. Возвращает true, если строка конфигурации содержит описание нумераторов и его удалось успешно десериализовать; false в обратном случае.

```cpp
IEnumerable<INumeratorInfo> numeratorsCollection;
bool success = _parser.TryParseNumeratorDeclaration(configuration, out numeratorsCollection);
```
где:
 - `success ` - результат успешной десериализации описания нумераторов из приведенной строки конфигурации атрибута;
 - `_parser ` - экземляр объекта, реализующего интерфейс IAttributeFormatParser;
 - `configuration ` - строка конфигурации атрибута;
 - `numeratorsCollection ` - коллекция десериализованных описаний нумераторов, содержавшихся в приведенной строке конфигурации;


<a name="PreviewNumeratorFormat"/>
##### string PreviewNumeratorFormat(string configuration, Dictionary<string, string> values); #####

Применяет нумератор к текущему состоянию объекта и возвращает строку, которая может использоваться для предпросмотра значения атрибута, которое будет назначено сервером при создании объекта и применения к нему нумератора. Используется для визуализации значения атрибута в карточке создания нового объекта, которое будет назначено сервером.

```cpp
string attributeValuePreview = _parser.PreviewNumeratorFormat(configuration, objectAttributes);
```
где:
 - `attributeValuePreview  ` - значение атрибута, которое будет получено на стороне сервера, при применении нумератора к текущему состоянию;

объекта;
 - `configuration ` - строка конфигурации нумератора;
 - `objectAttributes ` - текущее состояние объекта: коллекция пар <ИмяАтрибута, ЗначениеАтрибута>;


<a name="TryParseReferenceBookConfiguration"/>
##### bool TryParseReferenceBookConfiguration(string configuration, out IReferenceBookConfiguration referenceBookConfiguration); #####

Метод для получения десериализованного описания справочника из строки конфигурации атрибута. Возвращает true, если строка конфигурации содержит описание справочника и его удалось успешно десериализовать; false в обратном случае.

```cpp
IReferenceBookConfiguration referenceBookConfiguration;
bool success = _parser.TryParseReferenceBookConfiguration(configuration, out referenceBookConfiguration);
```
где:
 - `success ` - результат успешной десериализации описания справочника из приведенной строки конфигурации атрибута;
 - `_parser ` - экземляр объекта, реализующего интерфейс IAttributeFormatParser;
 - `configuration ` - строка конфигурации атрибута;
 - `numeratorsCollection ` - десериализованное описание справочника, содержавшееся в приведенной строке конфигурации;


<a name="INumeratorInfo"/>
### Интерфейс INumeratorInfo ###

Описание нумератора.

####string DisplayName { get; }####
Отображаемое имя нумератора.


####string Configuration { get; }####
Конфигурация нумератора, используется как параметр метода [PreviewNumeratorFormat](#PreviewNumeratorFormat), для получения предпросмотра значния атрибута, к которому применяется нумератор.

<a name="IReferenceBookConfiguration"/>
### Интерфейс IReferenceBookConfiguration ###

Конфигурация справочника, которая описывает его тип, поведение и правила формирования списка его элементов.

<a name="IReferenceBookConfiguration.Kind"/>
####RefBookKind Kind { get; }####
Свойство типа [RefBookKind](#RefBookKind), определяет тип справочника.

<a name="IReferenceBookConfiguration.Values"/>
####IEnumerable&lt;string&gt; Values { get; }####
Свойство, которое определяет коллекцию элементов справочника, если свойство [Kind](#IReferenceBookConfiguration.Kind) имеет значение [RefBookKind.Enum](#RefBookKind.Enum).

<a name="IReferenceBookConfiguration.Source"/>
####Guid Source { get; }####
Идентификатор объекта базы данных, дочерние объкты которого являются элементами справочника, если свойство [Kind](#IReferenceBookConfiguration.Kind) имеет значение [RefBookKind.Object](#RefBookKind.Object).


####IEnumerable&lt;string&gt; ElementsTypes { get; }####
Список имен типов, из объектов которого должны формироваться элементы справочника, если свойство [Kind](#IReferenceBookConfiguration.Kind) имеет значение [RefBookKind.Object](#RefBookKind.Object). Если имен список пуст, то в формировании элементов справочника учавствуют дочерние элементы любого типа.



####bool IsEditable { get; }####
Определяет, имеет ли пользователь возможность вручную отредактировать значение выбранного из элементов справочника. Если свойство имеет значение false, то редактор атрибута не допускает ручной ввод значений, и значение атрибута может быть лишь выбрано из списка. В противном случае, пользователь может вводить значение атрибута вручную, а список элементов справочника используется как список значений для быстрого заполнения.


####string StringFormat { get; }####
Строка форматирования, которая должна использоваться для формирования значения атрибута, при выборе элемента справочника.


<a name="RefBookKind"/>
### Перечисление RefBookKind ###

Указывает тип справочника, то есть один из возможных вариантов заполнения списка возможных значений справочника.

<a name="RefBookKind.Enum"/>
```cpp
RefBookKind.Enum;
```

Справочник, элементами которого является предопределенный набор строковых значений. Смотри свойство *Values* интерфеса *IReferenceBookConfiguration*.

<a name="RefBookKind.Object"/>
```cpp
RefBookKind.Object;
```

Справочник, элементами которого являются дочерние элементы определенного объекта в базе данных. Этот объект определяется идентификатором, значение которого возвращается свойством *Source* интерфеса *IReferenceBookConfiguration*.

```cpp
RefBookKind.OrgUnit;
```
Справочник, который предоставляет из выбор элементов оргструктуры базы данных.


<a name="RefBookEditorType"/>
### Перечисление RefBookEditorType ###

Указывает тип редактора, который должен использоваться для редактирования значения атрибута, для которого назначен справочник.

```cpp
RefBookEditorType.Dialog;
```
Редактор атрибута должен представлять собой поле ввода с кнопкой, которая вызывает новый диалог для выбора значений из элементов справочника.

```cpp
RefBookEditorType.ComboBox;
```

В качестве редактора значения атрибута должен использоваться комбобокс, выпадающий список которого состоит из элементов справочника.

<a name="CommonSettings"/>
Управление общими настройками
--------------------------------------------------
Общие настройки представляют собой персонифицированное хранилище типа "ключ-значение", где ключ - название настройки, а значение - ее значение. Список ключей системных настроек описан в классе [SystemSettingsKeys](#SystemSettingsKeys). Данные хранилища общих настроек синхронизируются с сервером. Значения настроек могут быть записаны как для конкретного пользователя, так и для элемента организационной структуры, причем настройки, записанные для конкретного пользователя, имеют приоритет перед настройками для элемента организационной структуры, в который входит должность, на которой состоит пользователь.

<a name="ISettingsFeature"/>
### Интерфейс ISettingsFeature ###
Интерфейс ISettingsFeature позволяет зарегистрировать в диалоге общих настроек новый ключ. Класс, реализующий данный интерфейс, должен быть помечен атрибутом [Export(typeof(ISettingsFeature))].

####string Key { get; }####
Ключ настройки.

####string Title { get; }####
Отображаемое имя настройки.

<a name="ISettingsFeatureEditor"/>
####FrameworkElement Editor { get; }####
Визуальный элемент, используемый для редактирования настроек данного типа.

####void SetValueProvider(ISettingValueProvider settingValueProvider)####
Данный метод вызывается со стороны приложения перед показом [Editor](#ISettingsFeatureEditor) и позволяет получить объект [ISettingValueProvider](#ISettingValueProvider) для редактирования значения настройки.

<a name="IPersonalSettings"/>
### Интерфейс IPersonalSettings ###
Импортируемый интерфейс, позволяющий получать и менять значения настроек текущего пользователя.

####IObservable<KeyValuePair<string, string>> SubscribeSetting(string key)####
Подписка на значение настройки по передаваемому ключу.

####void ChangeSettingValue(string key, string value)####
Изменение значения настройки по ключу.

<a name="ISettingValueProvider"/>
### Интерфейс ISettingValueProvider ###
Позволяет получать и устанавливать значение настройки для диалога общих настроек.

####string GetValue()####
Получение текущего значения настройки.

####void SetValue(string value)####
Установка значения настройки.

<a name="SystemSettingsKeys"/>
### Класс SystemSettingsKeys ###
Класс SystemSettingsKeys описывает ключи системных общих настроек.

####string FavoritesFeatureKey { get; }####
Список избранного в обозревавателе документов.

####string PilotStorageDriveLetter { get; }####
Предпочтительная буква монтирования диска Pilot-Storage.

####string AgreementRolesFeatureKey { get; }####
Список ролей для согласования.

####string MountedItemsList { get; }####
Список смонтированных проектов.

####string AutoBlockingFileExtensionsFeatureKey { get; }####
Список расширений файлов на Pilot-Storage, для которых будет работать функционал автоблокировки.

####string DocsAutoFillFeatureKey { get; }####
Настройки автозаполнения полей файлов.

<a name="SearchInterfaces"/>
Поиск
--------------------------------------------------

Поисковая система Pilot предоставляет богатый язык запросов. Построение запросов осуществляется с помощью конструкторов запросов (IQueryBuilder). Поиск можно осуществлять как по полям объектов так и по ключевому слову или части слова (фразы). Основными элементами языка запросов являются так называемые "[условия поиска](#MainSearchTerms)".
Чтобы осуществить поиск необходимо:
- получить конструктор поисковых запросов из интерфейса ISearchService.
- построить запрос с помощью полученного конструктора поисковых запросов с использованием "условий поиска".
- выполнить поиск вызвав метод Search и передать конструктор поисковых запросов в качестве параметра.

Конструкторы запросов имеют три ключевых метода для построения поисковых запросов:
 - `Must` - заданное условие обязательно должно выполниться.
 - `MustAnyOf` - должно выполняться одно из заданных условий.
 - `MustNot` - заданное условие не должно выполниться.

Чтобы задать поиск по определенному полю объекта используйте условия поиска для объектов [ObjectFields](#ObjectFields) и конструктор запросов для объектов (см [GetObjectQueryBuilder](#ObjectQueryBuilder))
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.TypeId.BeAnyOf(16,17));
```
Чтобы задать поиск по определенному полю задания используйте условия поиска для заданий [TaskFields](#TaskFields) и конструктор запросов для заданий (см [GetTaskQueryBuilder](#TaskQueryBuilder))
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.State.Be(State.InProgress));
```
Для любых объектов в базе Pilot предусмотрен поиск по ключевым словам. Такой поиск осуществляется с помощью **"условия поиска AllText"**.
Поиск по ключевому слову:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.AllText.Be("Проект*"));
```
Поиск по ключевым словам поддерживает одиночный (?) и многократный (*) подстановочные символы.

Для поиска с использованием одного символа подстановки используется служебный символ `?`. Например, чтобы найти слова "test" и "text" вы можете использовать следующую запись:
```cs
ObjectFields.AllText.Be("te?t")
```
Для поиска с использованием многократного символа подстановки используется служебный символ `*`.  Например, чтобы найти слова "test", "tests" или "tester" вы можете использовать следующую запись:
```cs
ObjectFields.AllText.Be("test*")
```
Также есть возможность использования символа многократной подстановки в середине или в начале слова. Например:
```cs
ObjectFields.AllText.Be("te*t")
```
**ВНИМАНИЕ!**
Не рекомендуется использовать подстановочные символы в начале слова. Это приведет к полному  перебору поисковых индексов, что может привести к сильным задержкам выдачи результата.



<a name="ISearchService"/>
### Интерфейс ISearchService ###

Интерфейс работы с поисковыми запросами.
Для получения доступа к интерфейсу поиска его необходимо добавить как параметр в конструктор класса плагина. При этом конструктор обязательно необходимо пометить атрибутом `[ImportingConstructor]`.

Чтобы осуществить поиск объектов необходимо предварительно составить поисковый запрос с помощью выбранного конструктора запроса. После задания условий в конструкторе запросов с помощью [условий поиска](#MainSearchTerms), его необходимо передать как параметр в метод [Search](#Search). Существует четыре типа конструкторов поисковых запросов:
- конструктор запросов для объектов "Обозревателя документов" (см [GetObjectQueryBuilder](#ObjectQueryBuilder));
- конструктор запросов для заданий (см [GetTaskQueryBuilder](#TaskQueryBuilder));
- универсальный конструктор запросов (см [GetEmptyQueryBuilder](#EmptyQueryBuilder));
- конструктор поисковой строки для "умной папки" (см [GetSmartFolderQueryBuilder](#SmartFolderQueryBuilder))

<a name="Search"/>
#### IObservable< ISearchResult> Search(IQueryBuilder query) ####

С помощью этого метода осуществляется поиск объектов по заданным критериям в конструкторе запросов. Результатом вызова этого метода является поставщик push-уведомлений результатов поиска cм. [ISearchResult](#ISearchResult).

```cpp
IObservable<ISearchResult> Search(IQueryBuilder query);
```

<a name="ObjectQueryBuilder"/>
#### IQueryBuilder GetObjectQueryBuilder() ####

Метод получения предварительно сконфигурированного конструктора запросов для пользовательских объектов.
В результат поисковой выдачи не попадают:
  - объекты системных и сервисных типов;
  - удаленные объекты.

```cpp
IQueryBuilder ISearchService.GetObjectQueryBuilder();
```

<a name="TaskQueryBuilder"/>
#### IQueryBuilder GetTaskQueryBuilder() ####

Метод получения предварительно сконфигурированного конструктора запросов для заданий.
В результат поисковой выдачи не попадают:
  - версии заданий;
  - задания в состоянии None.

```cpp
IQueryBuilder ISearchService.GetTaskQueryBuilder();
```

<a name="EmptyQueryBuilder"/>
#### IQueryBuilder GetEmptyQueryBuilder() ####

Метод получения простого конструктора запросов. Это универсальный конструктор запросов без предварительных условий. Этот конструктор можно использовать для поиска любых объектов в базе, включая объекты системных типов, например файл или папка на Pilot-Storage.

```cpp
IQueryBuilder ISearchService.GetEmptyQueryBuilder();
```

<a name="SmartFolderQueryBuilder"/>
##### ISearchService.GetSmartFolderQueryBuilder #####

Метод получения конструктора строки запроса для `умной папки`.

```cpp
ISmartFolderQueryBuilder ISearchService.GetSmartFolderQueryBuilder();
```

<a name="ISmartFolderQueryBuilder"/>
### Интерфейс ISmartFolderQueryBuilder ###

Интерфейс создания поискового запроса `умной папки`.


##### ISmartFolderQueryBuilder.WithSearchMode #####

Задает режим поиска (по атрибутам или по файлам)

```cpp
ISmartFolderQueryBuilder WithSearchMode(SearchMode searchMode);
```
где:
 - `searchMode` - режим поиска.


##### ISmartFolderQueryBuilder.WithKeyword #####

Задает ключевое слово для поиска

```cpp
ISmartFolderQueryBuilder WithKeyword(string keyword);
```
где:
 - `keyword` - ключевое слово.


##### ISmartFolderQueryBuilder.WithQuotedKeyword #####

Задает ключевое слово для точного поиска.

```cpp
ISmartFolderQueryBuilder WithQuotedKeyword(string keyword);
```
где:
 - `keyword` - ключевое слово.


##### ISmartFolderQueryBuilder.WithType #####

Задает тип элемента для поиска.

```cpp
ISmartFolderQueryBuilder WithType(int typeId);
```
где:
 - `typeId` - идентификатор типа.

##### ISmartFolderQueryBuilder.WithTypes #####

Задает список типов элементов для поиска.

```cpp
ISmartFolderQueryBuilder WithTypes(IEnumerable<int> typeIds);
```
где:
 - `typeIds` - список идентификаторов типов.

##### ISmartFolderQueryBuilder.WithState #####

Задает состояние элементов для поиска.

```cpp
ISmartFolderQueryBuilder WithState(ObjectState state);
```
где:
 - `state` - состояние объекта.

##### ISmartFolderQueryBuilder.WithAuthor #####

Задает автора (создателя) элемента для поиска.

```cpp
ISmartFolderQueryBuilder WithAuthor(int authorId);
```
где:
 - `authorId` - идентификатор пользователя.



##### ISmartFolderQueryBuilder.WithAuthors #####

Задает авторов (создателей) элементов для поиска.

```cpp
ISmartFolderQueryBuilder WithAuthors(IEnumerable<int> authorIds);
```
где:
 - `authorIds` - список идентификаторов пользователя.


##### ISmartFolderQueryBuilder.WithCreatedToday #####

Задает время создания элементов "сегодня" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedToday();
```

##### ISmartFolderQueryBuilder.WithCreatedYesterday #####

Задает время создания элементов "вчера" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedYesterday();
```

##### ISmartFolderQueryBuilder.WithCreatedThisWeek #####

Задает время создания элементов "на этой неделе" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedThisWeek();
```

##### ISmartFolderQueryBuilder.WithCreatedLastWeek #####

Задает время создания элементов "на прошлой неделе" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedLastWeek();
```

##### ISmartFolderQueryBuilder.WithCreatedThisMonth #####

Задает время создания элементов "в этом месяце" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedThisMonth();
```


##### ISmartFolderQueryBuilder.WithCreatedLastMonth #####

Задает время создания элементов "в прошлом месяце" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedLastMonth();
```


##### ISmartFolderQueryBuilder.WithCreatedThisYear #####

Задает время создания элементов "в этом году" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedLastYear();
```


##### ISmartFolderQueryBuilder.WithCreatedLastYear #####

Задает время создания элементов "в прошлом году" для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedThisYear();
```


##### ISmartFolderQueryBuilder.WithCreatedInRange #####

Задает диапазон дат создания элементов для поиска.

```cpp
ISmartFolderQueryBuilder WithCreatedInRange(DateTime fromUtc, DateTime toUtc);
```
где:
 - `fromUtc` - начальная дата в UTC.
 - `toUtc` - конечная дата  в UTC.


##### ISmartFolderQueryBuilder.ToString #####

Преобразует поисковый запрос в строку для последующего сохранения ее в соответствующем атрибуте `умной папки`

```cpp
string ToString();
```
<a name="SmartFolderCreationExample"/>
#####Пример создания "умной папки" #####

```cpp
using Ascon.Pilot.SDK;

namespace SmartFolderSample
{
    [Export(typeof(IDataPlugin))]
    public class SmartFolderSample : IDataPlugin
    {
        private readonly ISearchService _searchService;
        private readonly IObjectsRepository _repository;
        private readonly IObjectModifier _modifier;

        [ImportingConstructor]
        public SmartFolderSample(ISearchService searchService, IObjectsRepository repository, IObjectModifier modifier)
        {
            _searchService = searchService;
            _repository = repository;
            _modifier = modifier;
        }

        public void CreateSmartFolder(IDataObject parent)
        {
            _modifier.Create(parent, GetSmartFolderType())
                     .SetAttribute(SystemAttributeNames.SMART_FOLDER_TITLE, "title")
                     .SetAttribute(SystemAttributeNames.SEARCH_CRITERIA, GetSmartFolderSearchString())
                     .SetAttribute(SystemAttributeNames.SEARCH_CONTEXT_OBJECT_ID, parent.Id.ToString());
            _modifier.Apply();
        }

        private string GetSmartFolderSearchString()
        {
            var searchString = string.Empty;
            var type = _repository.GetType("Project");
            if (type == null)
                return searchString;

            var builder = _searchService.GetSmartFolderQueryBuilder();
            builder.WithSearchMode(SearchMode.Attributes)
                   .WithType(type.Id)
                   .WithKeyword("5000").ToString();
            searchString = builder.ToString();
            return searchString;
        }

        private IType GetSmartFolderType()
        {
            return _repository.GetType(SystemTypeNames.SMART_FOLDER);
        }
    }
}

```

<a name="IQueryBuilder"/>
### Интерфейс IQueryBuilder ###

Универсальный интерфейс конструктора поисковых запросов.


##### IQueryBuilder.Must #####

Используйте этот метод для [условий поиска](#MainSearchTerms), которые ДОЛЖНЫ быть в объектах.

```cpp
IQueryBuilder Must(ISearchTerm term);
```
где:
   - `term` - условие поиска. (например: ObjectFields.AllText.Be("Hello*"))


##### IQueryBuilder.MustAnyOf #####

Используйте этот метод, если какое-либо из указанных [условий поиска](#MainSearchTerms) ДОЛЖНО быть в объектах.

```cpp
IQueryBuilder MustAnyOf(params ISearchTerm[] terms);
```
где:
   - `terms` - условия поиска. (например: ObjectFields.TypeId.BeAnyOf(new[] {1, 2, 3})).

##### IQueryBuilder.MustNot #####

Используйте этот метод, если [условие поиска](#MainSearchTerms) НЕ ДОЛЖНО быть в объектах.

```cpp
IQueryBuilder MustNot(ISearchTerm term);
```
где:
   - `terms` - условие поиска. (например: TaskFields.State.Be(State.None)).


##### IQueryBuilder.SortBy #####

Используйте этот метод для того, чтобы установить поле и направление сортировки результатов поиска.

```cpp
IQueryBuilder SortBy(INamedField field, ListSortDirection direction);
```
где:
   - `field` - поле, по которому необходимо отсортировать результаты поиска (Например: ObjectFields.TypeId).
   - `direction` - направление сортировки.

   
<a name="IQueryBuilderMaxResults"/>
##### IQueryBuilder.MaxResults #####

Используйте этот метод для того, чтобы ограничить количество результатов. Значение по умолчанию 250.

```cpp
IQueryBuilder MaxResults(int result);
```
где:
   - `result` - максимальное количество результатов поиска.


##### IQueryBuilder.InContext #####

Используйте этот метод для того, чтобы установить контекст поиска.

```cpp
IQueryBuilder InContext(Guid id);
```
где:
   - `id` - идентификатор объекта.


<a name="ISearchResult"/>
### Интерфейс ISearchResult ###

Интерфейс предоставляет доступ к результатам поиска.

##### ISearchResult.Result #####

Список идентификаторов найденных объектов.

```cpp
IEnumerable<Guid> Result { get; }
```

##### ISearchResult.Total #####

Общее количество найденных объектов. Так как поисковая выдача ограничена с помощью [IQueryBuilder.MaxResults](#IQueryBuilderMaxResults), в общем случае значение Total может быть больше количества результатов Result. Если требуется найти больше объектов, установите большее значение [IQueryBuilder.MaxResults](#IQueryBuilderMaxResults) и перезапустите поиск.

```cpp
long Total { get; }
```

##### ISearchResult.Kind #####

Источник результата поиска.

```cpp
SearchResultKind Kind { get; }
```

##### Перечисление SearchResultKind #####

Источник результата поиска.

Результаты были получены из локального хранилища клиента.
```cpp
SearchResultKind.Local
```
Результаты были получены с удаленного сервера.
```cpp
SearchResultKind.Remote
```

<a name="MainSearchTerms"/>
### Предустановленные условия поиска ###

<a name="ObjectFields"/>
### Условия поиска для объектов ObjectFields ###

##### ObjectFields.Id #####

Используйте это условие, чтобы задать поиск по определенному идентификатору объекта.
Например:
```cs
var id  = new Guid("01A1028B-79BC-45C7-8A59-6191E8ADFE39");
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.Id.Be(id));
```

##### ObjectFields.ParentId #####

Используйте это условие, чтобы задать поиск по определенному идентификатору родительского объекта.
Например:
```cs
var id  = new Guid("01A1028B-79BC-45C7-8A59-6191E8ADFE39");
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.ParentId.Be(id));
```

##### ObjectFields.TypeId #####

Используйте это условие, чтобы задать поиск по типу объекта.
Например: зададим условие поиска для объектов типа 16 ИЛИ 17
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.TypeId.BeAnyOf(16,17));
```

##### ObjectFields.ObjectState #####

Используйте это условие, чтобы задать поиск по состоянию объекта.
Например: зададим условие поиска для объектов в состоянии заморозка
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.ObjectState.BeAnyOf((int)ObjectState.Frozen));
```

Дргугой пример, поиск объектов, находящихся в корзине:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.ObjectState.BeAnyOf((int)ObjectState.InRecycleBin));
```

Или наоборот: фильтрация объектов, помещенных в корзину, которые не должны отображаться в результатах поиска:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.ObjectState.BeAnyOf(new {(int)ObjectState.Frozen, (int)ObjectState.Alive}));
```

##### ObjectFields.CreatorId #####

Используйте это условие, чтобы задать поиск по создателю объекта.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.CreatorId.Be(25));
```

##### ObjectFields.CreatedDate #####

Используйте это условие, чтобы задать поиск по дате создания объектов. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(ObjectFields.CreatedDate.BeInRange(fromDate, toDate));
```

##### ObjectFields.IsSecret #####

Используйте это условие, чтобы искать скрытые или общедоступные объекты.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.IsSecret.Be(true));
```

##### ObjectFields.LockState #####

Используйте это условие, чтобы искать заблокированные объекты.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.LockState.Be(LockState.Accepted));
```

##### ObjectFields.LockPersonId #####

Используйте это условие, чтобы искать объекты заблокированные определенным пользователем.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.LockPersonId.Be(32));
```

##### ObjectFields.AllText #####

Используйте это условие, чтобы искать объекты по слову или части слова. Поиск осуществляется по атрибутам объекта.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(ObjectFields.AllText.Be("Проект*"));
```

<a name="TaskFields"/>
### Условия поиска для заданий TaskFields ###

##### TaskFields.Id #####

Используйте это условие, чтобы искать задания по определенному идентификатору.
Например:
```cs
var taskId = new Guid("90242DF7-00A7-40E2-8424-071FFB2D8190");
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.Id.Be(taskId));
```

##### TaskFields.InitiatorPositionId #####

Используйте это условие, чтобы искать задания выданные пользователем.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.InitiatorPositionId.Be(42));
```
где: 42 - идентификатор должности пользователя.

##### TaskFields.ExecutorPositionId #####

Используйте это условие, чтобы искать задания полученные пользователем.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.ExecutorPositionId.Be(42));
```
где: 42 - идентификатор должности пользователя.

##### TaskFields.CreatedDate #####

Используйте это условие, чтобы искать задания созданные в заданный промежуток времени. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(TaskFields.CreatedDate.BeInRange(fromDate, toDate));
```

##### TaskFields.State #####

Используйте это условие, чтобы искать задания, находящиеся в определенных состояниях.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.State.BeAnyOf(State.Assigned, State.Completed, State.InProgress));
```

##### TaskFields.Title #####

Используйте это условие, чтобы искать задания с заданной темой.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.Title.ContainsAll("Задание на *", "КЖ"));
```

##### TaskFields.Description #####

Используйте это условие, чтобы искать задания с заданным описанием.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.Description.Be("Выполнить *"));
```

##### TaskFields.DeadlineDate #####

Используйте это условие, чтобы искать задания с заданным сроком выполнения. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(TaskFields.DeadlineDate.BeInRange(fromDate, toDate));
```

##### TaskFields.DateOfAssignment #####

Используйте это условие, чтобы искать задания выданные в заданный диапазон дат. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(TaskFields.DateOfAssignment.BeInRange(fromDate, toDate));
```

##### TaskFields.DateOfCompletion #####

Используйте это условие, чтобы искать задания выполненные в заданный диапазон дат. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(TaskFields.DateOfCompletion.BeInRange(fromDate, toDate));
```

##### TaskFields.DateOfStart #####

Используйте это условие, чтобы искать задания взятые в работу в заданный диапазон дат. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(TaskFields.DateOfStart.BeInRange(fromDate, toDate));
```

##### TaskFields.Kind #####

Используйте это условие, чтобы искать по типу задания. Задание на согласование или ознакомление.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.Kind.Be(TaskKind.Agreement));
```

##### TaskFields.AllText #####

Используйте это условие, чтобы искать задания по слову или части слова. Поиск осуществляется по атрибутам задания.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.AllText.Be("Задание 1-*"));
```

##### TaskFields.TaskIsVersion #####

Используйте это условие, чтобы искать версии задания или исключить их.
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.TaskIsVersion.Be(true));
```

<a name="AttributeFields"/>
### Условия поиска по произвольным атрибутам AttributeFields###

##### AttributeFields.String #####

Используйте это условие, чтобы искать объекты по атрибуту типа строка.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(AttributeFields.String("AttributeName").Be("Искать*"));
```

##### AttributeFields.DateTime #####

Используйте это условие, чтобы искать объекты по атрибуту типа дата. Дату и время необходимо задавать в формате UTC.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(AttributeFields.DateTime("AttributeName").BeInRange(fromDate, toDate));
```

##### AttributeFields.Double #####

Используйте это условие, чтобы искать объекты по атрибуту типа вещественное число.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(AttributeFields.Double("AttributeName").Be(2.2));
```

##### AttributeFields.Integer #####

Используйте это условие, чтобы искать объекты по атрибуту типа целое число.
Например:
```cs
var builder = _searchService.GetObjectQueryBuilder();
builder.Must(AttributeFields.Integer("AttributeName").Be(2));
```


<a name="AllTerms"/>
### Общие условия поиска###

##### Условие Be #####

Используйте это условие для точного поиска значения.
```cs
ISearchTerm Be(T value);
```
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.ExecutorPositionId.Be(42));
```

##### Условие BeAnyOf #####

Поиск значений удовлетворяющих одному из условий.
```cs
ISearchTerm BeAnyOf(params T[] values);
```
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.ExecutorPositionId.BeAnyOf(45,42,15));
```

##### Условие BeInRange #####

Поиск значений находящихся в заданном диапазоне значений.
```cs
ISearchTerm BeInRange(T from, T to);
```
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
var fromDate = new DateTime(2016, 5, 5).ToUniversalTime();
var toDate = DateTime.Today.ToUniversalTime();
builder.Must(TaskFields.DateOfCompletion.BeInRange(fromDate, toDate));
```

##### Условие ContainsAll #####

Поиск значений соответствующих всем заданным условиям.
```cs
ISearchTerm ContainsAll(params T[] values);
```
Например:
```cs
var builder = _searchService.GetTaskQueryBuilder();
builder.Must(TaskFields.AllText.ContainsAll("задание", "согласование"));
```

<a name="ObjectSearchExample"/>
##### Пример создания поискового запроса #####

```cpp
using Ascon.Pilot.SDK;

namespace SearchSample
{
    [Export(typeof(IDataPlugin))]
    public class SearchSamplePlugin : IDataPlugin, IObserver<ISearchResult>
    {
        private readonly IObjectsRepository _repository;
        private long _total;

        [ImportingConstructor]
        public SearchSamplePlugin(ISearchService searchService, IObjectsRepository repository)
        {
            _repository = repository;
            var currentPerson = repository.GetCurrentPerson();
            var fromDate = new DateTime(2016,1,1);
            var to = new DateTime(2016,12,31);
            //Построим запрос на поиск объектов:
            // - созданные текущим пользователем
            // - объекты только типа "Project"
            // - созданные в 2016 году
            // - сортировать результаты по дате создания. По возрастанию.
            // - вывести только 50 результатов
            var qBuilder = searchService.GetObjectQueryBuilder();
            qBuilder.Must(ObjectFields.CreatorId.Be(currentPerson.Id))
                    .Must(ObjectFields.TypeId.Be(GetProjectTypeId()))
                    .Must(ObjectFields.CreatedDate.BeInRange(fromDate, to))
                    .SortBy(ObjectFields.CreatedDate, ListSortDirection.Ascending)
                    .MaxResults(50);

            searchService.Search(qBuilder).Subscribe(this);
        }

        public void OnNext(ISearchResult value)
        {
            var items = value.Result;
            _total = value.Total;
            _repository.SubscribeObjects(items).Subscribe(SomeObjectsLoader);
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }

        private int GetProjectTypeId()
        {
            //Сработает только если в конфигурации базы данных присутствует тип с именем Project
            var type = _repository.GetType("Project");
            if (type == null)
                throw new Exception("Type Project not found");

            return type.Id;
        }
    }
}

```

<a name="ObjectInterfaces"/>
Интерфейсы объектов
-------------------

К данным интерфейсам относятся объекты, которые содержат информацию об элементах, пользователях, заданиях и т.п.

Система взаимодействия расширений с клиентом Pilot построена на механизме доменов приложений. Код из одного домена не может получить прямой доступ к коду или данным другого. Взаимодействие между доменами обеспечивается исключительно посредством специальных прокси-классы. Все объекты, которые получает расширение являются прокси-классами, у которых есть время жизни. Все объекты-классы имеют время жизни по умолчанию равное 2 минуты. Если обратиться к прокси-классу после истечения времени его жизни, то бросится исключение типа System.Runtime.Remoting.RemotingException. 

Например:
System.Runtime.Remoting.RemotingException: Объект ‘76e7cd41_2cd2_4e89_9c03_fae752ec4d59
/zb_uualy_cm6kwizjlentfdl_3.rem’ был отключен или не существует на сервере.

Чтобы избежать появления этого исключения необходимо обернуть прокси-классы в так называемые классы-обертки. Например класс-обертка для прокси-класса реализующего интерфейс IDataObject может выглядеть так:
```cpp
	//Класс-обертка для IDataObject
	class DataObject
    {
        public DataObject(IDataObject dataObject)
        {
			//Запоминаем нужные для работы свойства dataObject
			Id = dataObject.Id;
            ParentId = dataObject.ParentId;
            Title = dataObject.DisplayName;
            Icon = dataObject.Type.SvgIcon;
            Sort = dataObject.Type.Sort;
            Type = new DataType(dataObject.Type);
            Children = new ReadOnlyCollection<Guid>(dataObject.Children);
            Attributes = new Dictionary<string, object>(dataObject.Attributes);
            Access = new ReadOnlyCollection<AccessRecord>(dataObject.Access2);
            Files = new ReadOnlyCollection<DataFile>(dataObject.Files.Select(f => new DataFile(f)).ToList());
            IsSecret = dataObject.IsSecret;
            Relations = new ReadOnlyCollection<Relation>(obj.Relations.Select(r => new Relation(r)));
        }

        public Guid Id { get; private set; }
        public Guid ParentId { get; private set; }
        public string Title { get; private set; }
        public byte[] Icon { get; private set; }
        public int Sort { get; private set; }
        public DataType Type { get; }
        public ReadOnlyCollection<Guid> Children { get; private set; }
        public IDictionary<string, object> Attributes { get; private set; }
        public ReadOnlyCollection<AccessRecord> Access { get; }
        public ReadOnlyCollection<DataFile> Files { get; private set; }
        public bool IsSecret { get; private set; }
        public ReadOnlyCollection<Relation> Relations { get; }
    }

    //Класс-обертка для IType
    public class DataType
    {
        public DataType(IType type)
        {
        	//Запомним нужные свойства type
            Id = type.Id;
            IsMountable = type.IsMountable;
            Attributes = new ReadOnlyCollection<AttributeWrapper>(type.Attributes.Select(x => new AttributeWrapper(x)).ToList());
            Title = type.Title;
            Name = type.Name;
            IsService = type.IsService;
            Kind = type.Kind;
            Sort = type.Sort;
            Children = new ReadOnlyCollection<int>(type.Children);
            HasFiles = type.HasFiles;
            Icon = type.SvgIcon;
        }

        public int Id { get; private set; }
        public bool IsMountable { get; private set; }
        public ReadOnlyCollection<AttributeWrapper> Attributes { get; private set; }
        public string Title { get; private set; }
        public string Name { get; private set; }
        public bool IsService { get; private set; }
        public TypeKind Kind { get; private set; }
        public int Sort { get; private set; }
        public ReadOnlyCollection<int> Children { get; private set; }
        public bool HasFiles { get; private set; }
        public byte[] Icon { get; private set; }
    }
    
    //Класс-обертка для IAttribute
    class AttributeWrapper
    {
    	public AttributeWrapper(IAttribute attribute)
        {
            Name = attribute.Name;
            Title = attribute.Title;
            IsObligatory = attribute.IsObligatory;
            DisplaySortOrder = attribute.DisplaySortOrder;
            ShowInObjectsExplorer = attribute.ShowInObjectsExplorer;
            IsService = attribute.IsService;
            Configuration = attribute.Configuration;
            DisplayHeight = attribute.DisplayHeight;
            Type = attribute.Type;
        }
        
        public string Name { get; private set; }
        public string Title { get; private set; }
        public AttributeType Type { get; private set; }
        public bool IsObligatory { get; private set; }
        public int DisplaySortOrder { get; private set; }
        public bool ShowInObjectsExplorer { get; private set; }
        public bool IsService { get; private set; }
        public string Configuration { get; private set; }
        public int DisplayHeight { get; private set; }
    }
    
    //. . . и т.д.
```

Данный пример применим, если необходимо полностью скопировать свойства прокси-класса. Если для работы расширения достаточно запомнить только идентификатор полученного объекта, то в создании класса-обертки нет необходимости.

<a name="IDataObject"/>
### Интерфейс IDataObject ###

Интерфейс предназначен для просмотра информации об элементе.


##### IDataObject.Id #####

Поле Id содержит идентификатор элемента.
```cpp
Guid IDataObject.Id { get; }
```


##### IDataObject.ParentId #####

Поле ParentId содержит идентификатор родительского элемента.

```cpp
Guid IDataObject.ParentIdId { get; }
```


##### IDataObject.Created #####

Поле Created содержит дату создания элемента.

```cpp
DateTime IDataObject.Created { get; }
```


##### IDataObject.Attributes #####

Поле Attributes содержит словарь атрибутов элемента. Где значением атрибута могут быть следующие типы: `string`, `int`, `DateTime` и `double`.

```cpp
IDictionary<string, object> IDataObject.Attributes { get; }
```


##### IDataObject.DisplayName #####

Поле DisplayName содержит отображаемое имя элемента, собранное из доступных атрибутов по определенному правилу.

```cpp
string IDataObject.DisplayName { get; }
```


##### IDataObject.Type #####

Поле Type содержит тип элемента.

```cpp
IType IDataObject.Type { get; }
```


##### IDataObject.Creator #####

Поле Creator содержит информацию о пользователе, создавшем текущий элемент.

```cpp
IPerson IDataObject.Creator { get; }
```


##### IDataObject.Children #####

Поле Children содержит список идентификаторов дочерних элементов.

```cpp
ReadOnlyCollection<Guid> IDataObject.Children { get; }
```

##### IDataObject.Relations #####

Поле Relations содержит список всех связей элемента с другими элементами.

```cpp
ReadOnlyCollection<IRelation> IDataObject.Relations { get; }
```

##### IDataObject.RelatedSourceFiles #####

Поле RelatedSourceFiles содержит список идентификаторов элементов, связанных с текущим связью типа "Исходный файл"

```cpp
ReadOnlyCollection<Guid> IDataObject.RelatedSourceFiles { get; }
```

##### IDataObject.RelatedTaskInitiatorAttachments #####

Поле RelatedTaskInitiatorAttachments содержит список идентификаторов элементов, связанных с текущим связью типа "Вложение в задание от инициатора"

```cpp
ReadOnlyCollection<Guid> IDataObject.RelatedTaskInitiatorAttachments { get; }
```

##### IDataObject.RelatedTaskMessageAttachments #####

Поле RelatedTaskMessageAttachments содержит список идентификаторов элементов, связанных с текущим связью типа "Вложение в сообщение по заданию"

```cpp
ReadOnlyCollection<Guid> IDataObject.RelatedTaskMessageAttachments { get; }
```

##### IDataObject.TypesByChildren #####

С помощью поле TypesByChildren можно определить тип ребенка элемента не загружая его.

```cpp
IDictionary<Guid, int> IDataObject.TypesByChildren { get; }
```

##### IDataObject.State #####

Поле State содержит информацию о текущем статусе элемента. Загружен элемент в кеш клиента или нет. Подробнее см. [DataState](#DataState)

```cpp
DataState IDataObject.State { get; }
```

##### IDataObject.ObjectStateInfo #####

Поле ObjectStateInfo содержит информацию о текущем статусе [ObjectState](#ObjectState) объекта и информацию о том, кто и когда совершил последнее изменения статуса. Подробнее см. [DataState](#DataState).

```cpp
DataState IDataObject.State { get; }
```

##### IDataObject.SynchronizationState #####

Поле SynchronizationState содержит информацию о текущем статусе синхронизации элемента с сервером. Подробнее см. [SynchronizationState](#SynchronizationState)

```cpp
SynchronizationState IDataObject.SynchronizationState { get; }
```

##### IDataObject.Files #####

Поле Files содержит список доступных файлов элемента. Содержит только информацию о файле. Тело файла необходимо загружать отдельно. Подробнее см. [IFileProvider](#IFileProvider)

```cpp
ReadOnlyCollection<IFile> IDataObject.Files { get; }
```

##### IDataObject.Access2 #####

Поле Access2 содержит информацию о текущих правах доступа к этому элементу. Подробнее см [IAccessRecord](#IAccessRecord).

```cpp
ReadOnlyCollection<IAccessRecord> IDataObject.Access2 { get; }
```

##### IDataObject.IsSecret #####

Поле IsSecret содержит значение указывающее является ли этот элемент скрытым.

```cpp
 bool IDataObject.IsSecret { get; }
```

##### IDataObject.ActualFileSnapshot #####

Поле ActualFileSnapshot содержит информацию о текущих фалах элемента. Подробнее см. [IFilesSnapshot](#IFilesSnapshot).

```cpp
 IFilesSnapshot IDataObject.ActualFileSnapshot { get; }
```

##### IDataObject.PreviousFileSnapshots #####

Поле PreviousFileSnapshots содержит информацию об изменениях файлов элемента. Это история изменения файлов элемента. Подробнее см. [IFilesSnapshot](#IFilesSnapshot).

```cpp
 ReadOnlyCollection<IFilesSnapshot> IDataObject.PreviousFileSnapshots { get; }
```

##### IDataObject.Subscribers #####

Поле Subscribers содержит список пользователей, которые подписаны на изменения этого элемента.

```cpp
 ReadOnlyCollection<int> IDataObject.Subscribers { get; }
```

##### IDataObject.ToTaskObject #####
Используется для конвертации текущего экземпляра IDataObject в экземпляр ITaskObject. Если текущий экземпляр IDataObject не является объектом типа задание, метод возвращает null.
```cs
IDataObject ToDataObject();
```

<a name="ITaskObject"/>
### Интерфейс ITaskObject ###
Интерфейс предназначен для просмотра информации о задании.

##### ITaskObject.Id #####
Поле Id содержит идентификатор задания.
```cs
Guid ITaskObject.Id { get; }
```

##### ITaskObject.Initiator #####
Поле Initiator содержит информацию об инициаторе задания.
```cs
IPerson Initiator { get; }
```

##### ITaskObject.Executor #####
Поле Executor содержит информацию об исполнителе задания.
```cs
IPerson Executor { get; }
```

##### ITaskObject.Created #####
Поле Created содержит дату создания задания.
```cs
DateTime ITaskObject.Created { get; }
```

##### ITaskObject.Children #####
Поле Children содержит список идентификаторов дочерних элементов, таких как версии задания.
```cpp
ReadOnlyCollection<Guid> ITaskObject.Children { get; }
```

##### ITaskObject.Attributes #####
Поле Attributes содержит словарь атрибутов задания. Где значением атрибута могут быть следующие типы: `string`, `int`, `DateTime` и `double`.
```cs
IDictionary<string, object> ITaskObject.Attributes { get; }
```

##### ITaskObject.Children #####
Поле Children содержит список идентификаторов детей задания (версий).
```cs
ReadOnlyCollection<Guid> Children { get; }
```

##### ITaskObject.InitiatorAttachments #####
Поле InitiatorAttachments содержит список идентификаторов вложений в задание, сделанных инициатором задания.
```cs
ReadOnlyCollection<Guid> InitiatorAttachments { get; }
```

##### ITaskObject.ExecutorAttachments #####
Поле ExecutorAttachments содержит список идентификаторов вложений в задание, сделанных исполнителем задания.
```cs
ReadOnlyCollection<Guid> ExecutorAttachments { get; }
```

##### ITaskObject.State #####
Поле State содержит текущее состояние задания. Подробнее см. [State](#State)
```cs
State State { get; }
```

##### ITaskObject.DataState #####
Поле DataState содержит информацию о текущем статусе элемента задания. Загружено задание в кеш клиента или нет. Подробнее см. [DataState](#DataState)
```cs
DataState DataState { get; }
```

##### ITaskObject.Title #####
Поле Title содержит тему задания.
```cs
string Title { get; }
```

##### ITaskObject.Description #####
Поле Description содержит описание задания.
```cs
string Description { get; }
```

##### ITaskObject.DeadlineDate #####
Поле DeadlineDate содержит дату окончания работ по заданию. Если дата не указана, то значение этого поля равно DateTime.MaxValue.
```cs
DateTime DeadlineDate { get; }
```

##### ITaskObject.DateOfAssignment #####
Поле DateOfAssignment содержит дату выдачи задания.
```cs
DateTime DateOfAssignment { get; }
```

##### ITaskObject.DateOfCompletion #####
Поле DateOfCompletion содержит дату завершения работ по заданию.
```cs
DateTime? DateOfCompletion { get; }
```

##### ITaskObject.DateOfStart #####
Поле DateOfStart содержит дату начала работ по заданию.
```cs
DateTime? DateOfStart { get; }
```

##### ITaskObject.DisplayTitle #####
Поле DisplayTitle - вспомогательное свойство, которое возвращает тему задания или описание задания, если тема не задана.
```cs
string DisplayTitle { get; }
```

##### ITaskObject.SynchronizationState #####
Поле SynchronizationState содержит информацию о текущем статусе синхронизации задания с сервером. Подробнее см. [SynchronizationState](#SynchronizationState)
```cs
SynchronizationState SynchronizationState { get; }
```

##### ITaskObject.ChatId #####
Поле ChatId содержит идентификатор объекта переписки.
```cs
Guid ChatId { get; }
```

##### ITaskObject.IsVersion #####
Флаг IsVersion указывает, является ли текущее задание версией.
```cs
bool IsVersion { get; }
```

##### ITaskObject.Auditors #####
Поле Auditors содержит список организационных единиц аудиторов задания
```cs
IEnumerable<IOrganisationUnit> Auditors { get; }
```

##### ITaskObject.Tags #####
Поле Tags содержит список тэгов задания.
```cs
ReadOnlyCollection<string> Tags { get; }
```

##### ITaskObject.Kind #####
Поле Kind содержит тип задания. Подробнее см. [TaskKind](#TaskKind)
```cs
TaskKind Kind { get; }
```

##### ITaskObject.ToDataObject #####
Используется для конвертации текущего экземпляра ITaskObject в экземпляр IDataObject
```cs
IDataObject ToDataObject();
```

<a name="IWorkflowObject"/>
### Интерфейс IWorkflowObject ###
Интерфейс предназначен для просмотра информации о рабочем процессе заданий.

##### IWorkflowObject.Id #####
Поле Id содержит идентификатор рабочего процесса.
```cs
Guid Id { get; }
```

##### IWorkflowObject.Title #####
Поле Title содержит заголовок рабочего процесса.
```cs
string Title { get; }
```

##### IWorkflowObject.Description #####
Поле Description содержит описание рабочего процесса.
```cs
string Description { get; }
```

##### IWorkflowObject.InitiatorAttachments #####
Поле InitiatorAttachments содержит список вложений инициатора рабочего процесса.
```cs
IEnumerable<Guid> InitiatorAttachments { get; }
```

<a name="IStageObject"/>
### Интерфейс IStageObject ###
Интерфейс предназначен для просмотра информации об этапе рабочего процесса заданий.

##### IStageObject.Id #####
Поле Id содержит идентификатор этапа.
```cs
Guid Id { get; }
```

##### IStageObject.Order #####
Поле Order содержит порядковый номер этапа.
```cs
int Order { get; }
```


<a name="IType"/>
### Интерфейс IType ###

Интерфейс предназначен для просмотра информации о типе элемента.


##### IType.Id #####

Поле Id содержит идентификатор типа элемента.
```cpp
Guid IType.Id { get; }
```


##### IType.Name #####

Поле Name содержит внутренне имя типа элемента.
```cpp
string IType.Name { get; }
```


##### IType.Title #####

Поле Title содержит локализованное имя типа элемента.
```cpp
string IType.Title { get; }
```


##### IType.Sort #####

Поле Sort содержит порядковый номер сортировки.
```cpp
int IType.Sort { get; }
```


##### IType.HasFiles #####

Поле HasFiles показывает может ли данный тип содержать файлы.
```cpp
bool IType.HasFiles { get; }
```


##### IType.Children #####

Поле Children содержит идентификаторы дочерних типов.
```cpp
ReadOnlyCollection<int> IType.Children { get; }
```


##### IType.Attributes #####

Поле Attributes содержит информацию об атрибутах, которые может содержать элемент данного типа.
```cpp
ReadOnlyCollection<IAttribute> IType.Attributes { get; }
```


##### IType.DisplayAttributes #####

Поле DisplayAttributes содержит информацию об отображаемых в *Обозревателе проектов* атрибутах.
```cpp
ReadOnlyCollection<IAttribute> IType.DisplayAttributes { get; }
```


##### IType.SvgIcon #####

Поле SvgIcon содержит массив байт иконки типа в формате svg.
```cpp
byte[] IType.SvgIcon { get; }
```


##### IType.IsMountable #####

Флаг IsMountable указывает может ли элемент данного типа монтироваться на диск (Pilot - Storage).
```cpp
bool IType.IsMountable { get; }
```


##### IType.Kind #####

Поле Kind содержит вид типа элемента.
```cpp
TypeKind IType.Kind { get; }
```


##### IType.IsDeleted #####

Поле IsDeleted указывает удален ли текущий тип. В системе **Pilot** типы не удаляются из базы данных, а только помечаются как удаленные.
```cpp
bool IType.IsDeleted { get; }
```


##### IType.IsService #####

Поле IsService указывает является ли текущий тип сервисным. Если данное свойство установлено в `true`, то элементы такого типа не будут отображаться в *Обозревателе проектов*. Доступ к элементам такого типа можно получить только из кода.
```cpp
bool IType.IsService { get; }
```

<a name="IAttribute"/>
### Интерфейс IAttribute ###

Интерфейс предназначен для просмотра информации об атрибутах типа элемента.


##### IAttribute.Name #####

Поле Name содержит внутреннее название атрибута типа.
```cpp
string IAttribute.Name { get; }
```


##### IAttribute.Title #####

Поле Title содержит локализованное название атрибута типа.
```cpp
string IAttribute.Title { get; }
```


##### IAttribute.IsObligatory #####

Поле IsObligatory показывает является ли данный атрибут обязательным для заполнения при создании нового элемента.
```cpp
bool IAttribute.IsObligatory { get; }
```

##### IAttribute.DisplaySortOrder #####

Поле DisplaySortOrder содержит порядковый номер сортировки атрибута типа.
```cpp
int IAttribute.DisplaySortOrder { get; }
```


##### IAttribute.ShowInObjectsExplorer #####

Поле ShowInObjectsExplorer показывает будет ли данный атрибут виден в *Обозревателе проектов*.
```cpp
bool IAttribute.ShowInObjectsExplorer { get; }
```

##### IAttribute.InGroup #####

Поле InGroup показывает, что, при наличии свободного места, данный атрибут группируется с соседними атрибутами, помечеными так же, при отображении в карточке объекта.
```cpp
bool IAttribute.InGroup { get; }
```


##### IAttribute.IsService #####

Поле IsService показывает является ли данный атрибут сервисным.
```cpp
bool IAttribute.IsService { get; }
```


##### IAttribute.Configuration #####

Поле Configuration содержит дополнительную информацию типа. Например этот атрибут используется для описания меток штрих-кода или описания справочников.
```cpp
string IAttribute.Configuration { get; }
```


##### IAttribute.DisplayHeight #####

Поле DisplayHeight содержит высоту атрибута, которую занимает атрибут в карточке редактирования и просмотра свойств элемента в *Обозревателе проектов*. По умолчанию атрибуты занимают одну строку.
```cpp
int IAttribute.DisplayHeight { get; }
```

<a name="IFile"/>
### Интерфейс IFile ###

Интерфейс предназначен для просмотра информации о файлах, которые содержит элемент.


##### IFile.Id #####

Поле Id содержит идентификатор файла
```cpp
Guid IFile.Id { get; }
```


##### IFile.Size #####

Поле Size содержит информацию о размере файла.
```cpp
long IFile.Size { get; }
```


##### IFile.Md5 #####

Поле Md5 содержит хеш-код файла, вычисленный по алгоритму хеширования md5
```cpp
string IFile.Md5 { get; }
```


##### IFile.Name #####

Поле Name содержит имя файла с расширением.
```cpp
string IFile.Name { get; }
```


##### IFile.Modified #####

Поле Modified содержит время последнего изменения файла.
```cpp
DateTime IFile.Modified { get; }
```


##### IFile.Created #####

Поле Created содержит время создания файла.
```cpp
DateTime IFile.Created { get; }
```


##### IFile.Accessed #####

Поле Accessed содержит время последнего доступа к файлу.
```cpp
DateTime IFile.Accessed { get; }
```


##### IFile.Signatures #####

Поле Signatures содержит список цифровых подписей, поставленных на файл.
```cpp
ReadOnlyCollection<ISignature> IFile.Signatures { get; }
```


<a name="ISignature"/>
### Интерфейс ISignature ###

Интерфейс предназначен для просмотра информации о цифровых подписях, поставленных на файл.


##### ISignature.Id #####

Поле Id содержит идентификатор цифровой подписи.
```cpp
Guid ISignature.Id { get; }
```


##### ISignature.DatabaseId #####

Поле DatabaseId содержит идентификатор базы данных, к которой привязана должность пользователя.
```cpp
Guid ISignature.DatabaseId { get; }
```


##### ISignature.PositionId #####

Поле PositionId содержит идентификатор должности пользователя, который подписал или должен подписать документ.
```cpp
int ISignature.PositionId { get; }
```


##### ISignature.Role #####

Поле Role содержит информацию о роли пользователя, который подписал или должен подписать документ.
```cpp
string ISignature.Role { get; }
```


##### ISignature.Sign #####

Поле Sign содержит информацию о подписи.
```cpp
string ISignature.Sign { get; }
```


##### ISignature.RequestedSigner #####

Поле RequestedSigner содержит информацию о должности пользователя, который должен подписать документ.
```cpp
string ISignature.RequestedSigner { get; }
```


<a name="IOrganisationUnit"/>
### Интерфейс IOrganisationUnit ###

Интерфейс предназначен для просмотра информации о должностях и структурных подразделениях организации.


##### IOrganisationUnit.Id #####

Поле Id содержит идентификатор должности или структурного подразделения.
```cpp
int IOrganisationUnit.Id { get; }
```


##### IOrganisationUnit.Title #####

Поле Title содержит локализованное наименование должности или структурного подразделения.
```cpp
string IOrganisationUnit.Title { get; }
```


##### IOrganisationUnit.IsDeleted #####

Поле IsDeleted показывает удалена ли должность или структурное подразделение. В системе *Pilot* организационные единицы не удаляются из базы данных, а только помечаются как удаленные.
```cpp
bool IOrganisationUnit.IsDeleted { get; }
```


##### IOrganisationUnit.Children #####

Поле Children содержит идентификаторы дочерних должностей или структурных подразделений.
```cpp
ReadOnlyCollection<int> IOrganisationUnit.Children { get; }
```


<a name="IPerson"/>
### Интерфейс IPerson ###

Интерфейс предназначен для просмотра информации о пользователе системы *Pilot*.


##### IPerson.Id #####

Поле Id содержит уникальный идентификатор пользователя.
```cpp
int IPerson.Id { get; }
```


##### IPerson.Login #####

Поле Login содержит имя пользователя (login).
```cpp
string IPerson.Login { get; }
```


##### IPerson.DisplayName #####

Поле DisplayName содержит отображаемое имя пользователя. Например ФИО.
```cpp
string IPerson.DisplayName { get; }
```


##### IPerson.Positions #####

Поле Positions содержит список должностей, которые в данный момент занимает пользователь.
```cpp
ReadOnlyCollection<IPosition> IPerson.Positions { get; }
```


##### IPerson.MainPosition #####

Поле MainPosition содержит основную должность пользователя.
```cpp
IPosition IPerson.MainPosition { get; }
```


##### IPerson.Comment #####

Поле Comment содержит дополнительные сведения о пользователе.
```cpp
string IPerson.Comment { get; }
```


##### IPerson.Sid #####

Поле Sid содержит уникальный идентификатор доменной учетной записи пользователя.
```cpp
string IPerson.Sid { get; }
```


##### IPerson.IsDeleted #####

Поле IsDeleted показывает удален пользователь или нет. В системе *Pilot* информация пользователях не удаляется из базы данных. Пользователи помечаются как удаленные.
```cpp
bool IPerson.IsDeleted { get; }
```


##### IPerson.IsAdmin #####

Поле IsAdmin показывает является ли текущий пользователь администратором.
```cpp
bool IPerson.IsAdmin { get; }
```


##### IPerson.ActualName #####

Поле ActualName содержит актуальное имя пользователя. По умолчанию это поле выводит информацию из поля `DisplayName`. Если поле `DisplayName` не заполнено, то данное поле выводит информацию из поля `Login`.
```cpp
string IPerson.ActualName { get; }
```


<a name="IPosition"/>
### Интерфейс IPosition ###

Интерфейс предназначен для просмотра информации о должности пользователя.


##### IPosition.Order #####

Поле Order содержит порядковый номер должности пользователя. Основная должность имеет наименьший порядковый номер.
```cpp
int IPosition.Order { get; }
```


##### IPosition.Position #####

Поле Position содержит идентификатор должности пользователя.
```cpp
int IPosition.Position { get; }
```


<a name="IAccess"/>
### Интерфейс IAccess ###

Интерфейс предназначен для просмотра информации об уровне доступа.


##### IAccess.AccessLevel #####

Содержит уровень доступа к элементу.

```cpp
AccessLevel IAccess.AccessLevel { get; }
```


##### IAccess.ValidThrough #####

Содержит дату, до которой действует право доступа.

```cpp
DateTime IAccess.ValidThrough { get; }
```


##### IAccess.IsInheritable #####

Флаг, указывающий унаследовано право от родительского элемента или нет.

```cpp
bool IAccess.IsInheritable { get; }
```


##### IAccess.IsInherited #####

Флаг, указывающий будут ли наследовать дочерние элементы текущее право доступа.

```cpp
bool IAccess.IsInherited { get; }
```

<a name="IAccessRecord"/>
### Интерфейс IAccessRecord ###
Интерфейс предназначен для просмотра информации о правах доступа на элемент.

##### IAccessRecord.OrgUnitId #####
Содержит идентификатор должности для которого действует указанный уровень доступа.
```cpp
int IAccessRecord.OrgUnitId { get; }
```

##### IAccessRecord.Access #####
Содержит информацию об уровене доступа. Подробнее см. [IAccess](#IAccess)
```cpp
IAccess IAccessRecord.Access { get; }
```

##### IAccessRecord.RecordOwner #####
Содержит информацию о должности, которая установила указанный уровень доступа.
```cpp
int IAccessRecord.RecordOwner { get; }
```

##### IAccessRecord.InheritanceSource #####
Содержит идентификатор элемента, с которого текущий элемент получил унаследованное право доступа.
```cpp
Guid IAccessRecord.InheritanceSource { get; }
```


<a name="IStorageDataObject"/>
### Интерфейс IStorageDataObject ###

Интерфейс предназначен для просмотра информации об элементе Pilot-Storage.


##### IStorageDataObject.DataObject #####

Поле DataObject содержит полную информацию об элементе.
```cpp
IDataObject IStorageDataObject.DataObject { get; }
```


##### IStorageDataObject.Path #####

Поле Path содержит абсолютный путь до элемента на Pilot-Storage.

```cpp
string IStorageDataObject.Path { get; }
```


##### IStorageDataObject.State #####

Поле State содержит текущее состояние элемента.

```cpp
StorageObjectState IStorageDataObject.State { get; }
```

##### IStorageDataObject.IsDirectory #####

Поле IsDirectory показывает является ли текущий элемент папкой.

```cpp
bool IStorageDataObject.IsDirectory { get; }
```

<a name="IStateInfo"/>
### Интерфейс IStateInfo ###

Интерфейс предназначен для просмотра информации о состоянии объекта.

##### IStateInfo.State #####

Поле State содержит ифнормацию о состоянии объекта.
```cpp
ObjectState IStateInfo.State { get; }
```

##### IStateInfo.Date #####

Поле Date содержит ифнормацию дате последнего изменения состояния объекта.
```cpp
DateTime IStateInfo.Date { get; }
```

##### IStateInfo.PersonId #####

Поле PersonId содержит идентификатор пользователя, изменившего состояние объекта последним.
```cpp
int IStateInfo.PersonId { get; }
```

##### IStateInfo.PositionId #####

Поле PositionId содержит идентификатор позиции пользователя, изменившего состояние объекта последним.
```cpp
int IStateInfo.PositionId { get; }
```

<a name="IRelation"/>
### Интерфейс IRelation ###

Интерфейс предоставляет доступ к свойствам связи элемента.

##### IRelation.Id #####

Поле Id содержит идентификатор связи. У двух связанных между собой объектов должны быть связи с одинаковым идентификатором.

```cpp
Guid IRelation.Id { get; }
```

##### IRelation.TargetId #####

Поле TargetId содержит идентификатор элемента, с которым связан текущий объект.

```cpp
Guid IRelation.TargetId { get; }
```

##### IRelation.Type #####

Поле Type содержит тип связи. Подробнее см [ObjectRelationType](#ObjectRelationType)

```cpp
ObjectRelationType IRelation.Type { get; }
```

##### IRelation.Name #####

Поле Name содержит имя связи.

```cpp
string IRelation.Name { get; }
```

##### IRelation.VersionId #####

Поле VersionId содержит дату создания версии документа.

```cpp
DateTime IRelation.VersionId { get; }
```

<a name="IFilesSnapshot"/>
### Интерфейс IFilesSnapshot ###

Интерфейс содержит информацию о файлах элемента.

##### IFilesSnapshot.Created #####
Поле Created содержит дату создания снимка файлов.
```cpp
DateTime IFilesSnapshot.Created { get; }
```

##### IFilesSnapshot.CreatorId #####
Поле CreatorId содержит идентификатор пользователя сделавшего снимок файлов.
```cpp
int IFilesSnapshot.CreatorId { get; }
```

##### IFilesSnapshot.Reason #####
Поле Reason содержит информацию о причине создания снимка файлов.
```cpp
string IFilesSnapshot.Reason { get; }
```

##### IFilesSnapshot.Files #####
Поле Files содержит список файлов снимка.
```cpp
ReadOnlyCollection<IFile> IFilesSnapshot.Files { get; }
```

<a name="Enums"/>
Перечисления
----------------------------------

<a name="DataState"/>
### Перечисление DataState ###
Перечисление состояний элемента.

##### DataState.Unknown #####
Не известное состояние элемента. Все создаваемые элементы получают данное состояние.
```cpp
DataState.Unknown;
```

##### DataState.NoData #####
Элемент существует, но не загружен в память клиента.
```cpp
DataState.NoData;
```

##### DataState.Loaded #####
Элемент существует и загружен в память клиента.
```cpp
DataState.Loaded;
```

##### DataState.NonExistent #####
Элемента не существует.
```cpp
DataState.NonExistent;
```

<a name="SynchronizationState"/>
### Перечисление SynchronizationState ###
Перечисление состояний синхронизации элемента с сервером.

##### SynchronizationState.Synchronized #####
Элемент синхронизирован с сервером. Элемент на сервере идентичен элементу в кеше клиента.
```cpp
SynchronizationState.Synchronized;
```

##### SynchronizationState.AwaitingForSynchronization #####
Элемент в ожидании синхронизации с сервером.
```cpp
SynchronizationState.AwaitingForSynchronization;
```

##### SynchronizationState.SynchronizationAborted #####
Изменения внесенные в элемент по каким-то причинам были отклонены сервером. Например отсутствие прав на изменение атрибутов элемента.
```cpp
SynchronizationState.SynchronizationAborted;
```

<a name="TypeKind"/>
### Перечисление TypeKind ###
Перечисление видов типа элемента.

##### TypeKind.User #####
Тип елемента является пользовательским и элементы данного типа могут отображаться в *Обозревателе проектов*.
```cpp
TypeKind.User;
```

##### TypeKind.System #####
Тип элемента является системным и элементы данного типа не отображаются в *Обозревателе проектов*.
```cpp
TypeKind.System;
```

<a name="ObjectState"/>
### Перечисление ObjectState ###
Перечисление состояний элемента.

##### ObjectState.Alive #####
Состояние по умолчанию. Элемент "живой".
```cpp
ObjectState.Alive;
```

##### ObjectState.InRecycleBin #####
Элемент удален в корзину
```cpp
ObjectState.InRecycleBin;
```

##### ObjectState.DeletedPermanently #####
Элемент удален безвозвратно
```cpp
ObjectState.DeletedPermanently;
```

##### ObjectState.Frozen #####
Элемент заморожен
```cpp
ObjectState.Frozen;
```


<a name="State"/>
### Перечисление State ###
Перечисление состояний элемента задания.

##### State.None #####
Черновик
```cpp
State.None;
```

##### State.Assigned #####
Задание выдано
```cpp
State.Assigned;
```

##### State.InProgress #####
Задание взято в работу
```cpp
State.InProgress;
```

##### State.Revoked #####
Задание отозвано
```cpp
State.Revoked;
```

##### State.OnValidation #####
Задание находится на проверке
```cpp
State.OnValidation;
```

##### State.Completed #####
Задание выполнено
```cpp
State.Completed;
```


<a name="AccessLevel"/>
### Перечисление AccessLevel ###

Перечисление возможных уровней доступа к элементам.


##### AccessLevel.None #####

Нет доступа к элементу.

```cpp
AccessLevel.None;
```


##### AccessLevel.Create #####

Уровень доступа, позволяющий создавать дочерние элементы.

```cpp
AccessLevel.Create;
```


##### AccessLevel.Edit #####

Уровень доступа, позволяющий редактировать текущий элемент.

```cpp
AccessLevel.Edit;
```


##### AccessLevel.View #####

Уровень доступа, позволяющий просматривать свойства текущего элемента.

```cpp
AccessLevel.View;
```


##### AccessLevel.ViewCreate #####

Уровень доступа, позволяющий просматривать свойства текущего элемента и создавать дочерние элементы. Это комбинация двух уровней доступа `Create` и `View`.

```cpp
AccessLevel.ViewCreate;
```
`


##### AccessLevel.ViewEdit #####

Полный доступ к элементу. Уровень доступа, позволяющий просматривать, редактировать свойства текущего элемента и создавать дочерние элементы. Это комбинация уровней доступа `Create`, `View` и `Edit`.

```cpp
AccessLevel.ViewEdit;
```


<a name="StorageObjectState"/>
### Перечисление StorageObjectState ###
Перечисление состояний элемента Pilot-Storage.

##### StorageObjectState.None #####
Неизвестное состояние элемента.
```cpp
StorageObjectState.None;
```

##### StorageObjectState.Loaded #####
Элемент загружен.
```cpp
StorageObjectState.Loaded;
```

##### StorageObjectState.Outdated #####
Элемент скачан с сервера. Но на сервере есть более новая версия.
```cpp
StorageObjectState.Outdated;
```

##### StorageObjectState.NotSent #####
Элемент еще не был отправлен на сервер.
```cpp
StorageObjectState.NotSent;
```

##### StorageObjectState.Edited #####
Есть локальные изменения, которые не отправлены на сервер.
```cpp
StorageObjectState.Edited;
```

##### StorageObjectState.Aborted #####
Изменения не принятые сервером.
```cpp
StorageObjectState.Aborted;
```

##### StorageObjectState.Temp #####
Временный элемент, который никогда не будет отправлен на сервер.
```cpp
StorageObjectState.Aborted;
```


<a name="ObjectRelationType"/>
### Перечисление ObjectRelationType ###

##### ObjectRelationType.SourceFiles #####
Тип связи с исходным файлом
```cpp
ObjectRelationType.SourceFiles;
```

##### ObjectRelationType.TaskInitiatorAttachments #####
Тип связи, описывающий вложения в задание документов или файлов инициатором задания.
```cpp
ObjectRelationType.TaskInitiatorAttachments;
```

##### ObjectRelationType.MessageAttachments #####
Тип связи, описывающий вложения в переписку по заданию документов или файлов.
```cpp
ObjectRelationType.MessageAttachments;
```

##### ObjectRelationType.Custom #####
Пользовательский тип связи. Этот тип связи отображается во вкладке "Связи" в клиенте Pilot.
```cpp
ObjectRelationType.Custom;
```

<a name="TaskKind"/>
### Перечисление TaskKind ###
Перечисление типов заданий.

##### TaskKind.Task #####
Обычное задание
```cpp
TaskKind.Task;
```

##### TaskKind.Agreement #####
Задание на согласование документов
```cpp
TaskKind.Agreement;
```

##### TaskKind.Acquaintance #####
Задание на ознакомление
```cpp
TaskKind.Acquaintance;
```



<a name="Events"/>
События
----------------------------------

<a name="OfflineEventArgs"/>
#### Событие OfflineEventArgs ####

Аргументы события `OfflineEventArgs` присылаются расширению когда возникает событие перехода клиента Pilot-ICE/ECM в режим автономной работы.
```cpp
public class OfflineEventArgs : MarshalByRefObject { }
```

<a name="OnlineEventArgs"/>
#### Событие OnlineEventArgs ####

Аргументы события `OnlineEventArgs` присылаются расширению когда возникает событие перехода клиента Pilot-ICE/ECM в режим автономной работы в сети.
```cpp
public class OnlineEventArgs : MarshalByRefObject { }
```

<a name="CloseTabEventArgs"/>
#### Событие CloseTabEventArgs ####

Аргументы события `CloseTabEventArgs` присылаются расширению перед закрытием вкладки.
```cpp
public class CloseTabEventArgs : MarshalByRefObject { }
```
Класс `CloseTabEventArgs` содержит поле с заголовком закрываемой вкладки.
```cpp
public string CloseTabEventArgs.TabTitle { get; private set; }
```

<a name="OpenTabEventArgs"/>
#### Событие OpenTabEventArgs ####

Аргументы события `OpenTabEventArgs` присылаются расширению перед открытием новой вкладки.
```cpp
public class OpenTabEventArgs : MarshalByRefObject { }
```
Класс `OpenTabEventArgs` содержит поле с заголовком открываемой вкладки.
```cpp
public string OpenTabEventArgs.TabTitle { get; private set; }
```

<a name="LoadedEventArgs"/>
#### Событие LoadedEventArgs ####

Аргументы события `LoadedEventArgs` присылаются расширению после загрузки клиентом Pilot-ICE/ECM данного расширения.
```cpp
public class LoadedEventArgs : MarshalByRefObject { }
```

<a name="UnloadedEventArgs"/>
#### Событие UnloadedEventArgs ####

Аргументы события `LoadedEventArgs` присылаются расширению перед выгрузкой клиентом Pilot-ICE/ECM данного расширения.
```cpp
public class UnloadedEventArgs : MarshalByRefObject { }
```



<a name="Additionally"/>
Дополнительно
-------------------------

<a name="NumeratorExample"/>
#### Пример создания объекта с нумератором ####
Для заполнения атрибута типа нумератор при создании объекта необходимо:

1. Получить метаданные атрибута создаваемого объекта. `IDataObject.Type.Attributes` -> найти по имени (или другому признаку) нужный атрибут;

2. Получить значение поля `IAttribute.Configuration` и распарсить его. Поле должно содержать описание возможных строк формата данного атрибута. Ниже приведен наиболее сложный пример такого описания:
`Входящий документ, ВХ-{Counterc11cbdead2d047399127798d2d1ea07f:d5}-{CurrentDate:yyyy}`
`Исходящий документ, ИС-{Counterc11cbdead2d047399127798d2d1ea07f:d5}-{CurrentDate:yyyy}`

3. Строками разделены различные возможные описания нумератора. Таких строк может быть 1 или более. Каждая строка содержит следующие данные, разделенные между собой знаком `","`:
	- Название описания (может быть пропущено)
	- Строка форматирования описания

4. Из распарсенных данных из пункта 2 выбираем нужное нам описание (если оно одно, то без выбора) и получаем значение его строки форматирования. Например, из вышеприведенного примера мы выберем
`ИС-{Counterc11cbdead2d047399127798d2d1ea07f:d5}-{CurrentDate:yyyy}`

5. Присваиваем значению атрибута создаваемого объекта значение строки форматирования из пункта 3.

6. Создаем объект, после прохождения объекта через сервер атрибут будет заполен согласно выбранной строке форматирования.

Пример:
```cpp
using System.Linq;
using System.Text;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.DataObjectWrappers;
using Ascon.Pilot.SDK.Menu;
using NumeratorPlugin.Wrappers;

namespace NumeratorPlugin
{
    [Export(typeof(IMenu<ObjectsViewContext>))]
    public class MainModule : IMenu<ObjectsViewContext>
    {
        private readonly IObjectsRepository _repository;
        private readonly IObjectModifier _modifier;
        private readonly IAttributeFormatParser _attributeFormatParser;
        private const string CREATE_COMMAND = "CreateCommand";

        [ImportingConstructor]
        public MainModule(IObjectsRepository repository, IObjectModifier modifier, IAttributeFormatParser attributeFormatParser)
        {
            _repository = repository;
            _modifier = modifier;
            _attributeFormatParser = attributeFormatParser;
        }

        public void Build(IMenuBuilder builder, ObjectsViewContext context)
        {
            builder.AddItem(CREATE_COMMAND, 0).WithHeader("Create element with numerator");
        }

        public void OnMenuItemClick(string name, ObjectsViewContext context)
        {
            var type = _repository.GetType("DocumentTypeWithNumerator");
            //PlgAttribute - wrapper class for the IAttribute
            var numberAttribute = new PlgAttribute(type.Attributes.First(att => att.Name == "Number"));
            IEnumerable<INumeratorInfo> numInfos = null;
            if (_attributeFormatParser.TryParseNumeratorDeclaration(numberAttribute.Configuration, out numInfos))
            {
            	//PlgDataObject - wrapper class for the IDataObject
                var parent = new PlgDataObject(context.SelectedObjects.FirstOrDefault());
                //PlgNumeratorInfo - wrapper class for the INumeratorInfo
                var infos = numInfos.Select(i => new PlgNumeratorInfo(i));
                foreach (var ni in infos)
                {
                    _modifier.Create(context.SelectedObjects.FirstOrDefault(), type).SetAttribute("Number", ni.Configuration);
                }
            }

            _modifier.Apply();
        }
    }
}
```