﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Media;
using Ascon.Pilot.SDK.Menu;
using Ascon.Pilot.Theme.ColorScheme;

namespace Ascon.Pilot.SDK.FileSnapshotSample
{
    [Export(typeof(IMenu<ObjectsViewContext>))]
    public class FileSnapshotSample : IMenu<ObjectsViewContext>
    {
        private readonly IObjectsRepository _repository;
        private readonly IObjectModifier _modifier;
        private Guid _selectedId;
        
        [ImportingConstructor]
        public FileSnapshotSample(IPilotDialogService dialogService, IObjectsRepository repository, IObjectModifier modifier)
        {
            _repository = repository;
            _modifier = modifier;
            var convertFromString = ColorConverter.ConvertFromString(dialogService.AccentColor);
            if (convertFromString != null)
            {
                var accentColor = (Color)convertFromString;
                ColorScheme.Initialize(accentColor, dialogService.Theme);
            }
        }

        public void Build(IMenuBuilder builder, ObjectsViewContext context)
        {
            var objects = context.SelectedObjects.ToList();
            if (objects.Count != 1)
                return;

            var current = objects.Single();
            if (!current.Type.HasFiles) 
                return;
            
            builder.AddItem("MenuCommand", 0).WithHeader("Show document versions");
            _selectedId = current.Id;
        }

        public void OnMenuItemClick(string name, ObjectsViewContext context)
        {
            var view = new DocumentHistoryView();
            var model = new DocumentHistoryViewModel(_selectedId, _repository, _modifier, view.Dispatcher);
            view.DataContext = model;
            view.ShowDialog();
        }
    }
}
