﻿using System;
using System.ComponentModel.Composition;
using System.Windows;

namespace Ascon.Pilot.SDK.LicenseSample
{
    [Export(typeof(IDataPlugin))]
    public class LicenseSampleMain : IDataPlugin, IObserver<bool>, IHandle<UnloadedEventArgs>
    {
        private readonly ILicenseManager _licenseManager;

        [ImportingConstructor]
        public LicenseSampleMain(ILicenseManager licenseManager, IEventAggregator eventAggregator)
        {
            _licenseManager = licenseManager;
            licenseManager.Consume(501).Subscribe(this);
            eventAggregator.Subscribe(this);
        }

        /// <summary>
        /// License  type handler
        /// </summary>
        /// <param name="licenseResult">true - valid license file exists, false - license files doesn't exist(free connections mode enabled)</param>
        public void OnNext(bool licenseResult)
        {
            if (licenseResult)
                MessageBox.Show("License captured");
            else
                MessageBox.Show("Free connections mode");
        }

        public void OnError(Exception error)
        {
            MessageBox.Show(error.Message);
        }

        public void OnCompleted()
        {
            MessageBox.Show("Action complete");
        }

        public void Handle(UnloadedEventArgs message)
        {
            _licenseManager.Release(501);
        }
    }
}
