﻿using System;
using System.ComponentModel.Composition;
using System.Windows;

namespace Ascon.Pilot.SDK.NotificationsSample
{
    [Export(typeof(IDataPlugin))]
    public class Main : IDataPlugin, IObserver<INotification>
    {
        private readonly IObjectsRepository _repository;

        [ImportingConstructor]
        public Main(IObjectsRepository repository)
        {
            _repository = repository;
            repository.SubscribeNotification(NotificationKind.ObjectCreated).Subscribe(this);
            repository.SubscribeNotification(NotificationKind.ObjectDeleted).Subscribe(this);
            repository.SubscribeNotification(NotificationKind.StorageObjectCreated).Subscribe(this);
            repository.SubscribeNotification(NotificationKind.StorageObjectDeleted).Subscribe(this);
        }

        public void OnNext(INotification value)
        {
            if (value.ChangeKind == NotificationKind.ObjectCreated)
            {
                var loader = new ObjectLoader(_repository);
                loader.Load(value.ObjectId, o =>
                {
                    var message = string.Format("Object {0} has been created", o.DisplayName);
                    MessageBox.Show(message, "Ascon.Pilot.SDK.NotificationsSample");    
                });
                return;
            }

            if (value.ChangeKind == NotificationKind.ObjectDeleted)
            {
                var loader = new ObjectLoader(_repository);
                loader.Load(value.ObjectId, o =>
                {
                    var message = string.Format("Object {0} has been deleted", o.DisplayName);
                    MessageBox.Show(message, "Ascon.Pilot.SDK.NotificationsSample");
                });
                return;
            }

            if (value.ChangeKind == NotificationKind.StorageObjectCreated)
            {
                var loader = new ObjectLoader(_repository);
                loader.Load(value.ObjectId, o =>
                {
                    var message = string.Format("Storage object {0} has been created", o.DisplayName);
                    MessageBox.Show(message, "Ascon.Pilot.SDK.NotificationsSample");
                });
                return;
            }

            if (value.ChangeKind == NotificationKind.StorageObjectDeleted)
            {
                var loader = new ObjectLoader(_repository);
                loader.Load(value.ObjectId, o =>
                {
                    var message = string.Format("Storage object {0} has been deleted", o.DisplayName);
                    MessageBox.Show(message, "Ascon.Pilot.SDK.NotificationsSample");
                });
            }
        }

        public void OnError(Exception error)
        {
            
        }

        public void OnCompleted()
        {
            
        }
    }
}
