﻿using System;
using System.Collections.ObjectModel;

namespace Ascon.Pilot.SDK.ObjectsSample.NodeViewModel
{
    class PersonNodeViewModel : PropertyChangedBase, IOrganizationUnitViewModel, IObserver<IPerson>
    {
        private PersonWrapper _person;
        private string _title;

        public PersonNodeViewModel(IPerson person, IObjectsRepository repository)
        {
            _person = new PersonWrapper(person);
            Title = _person.DisplayName;
            Children = new ObservableCollection<IOrganizationUnitViewModel>();
            repository.SubscribePeople().Subscribe(this);
        }

        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                NotifyPropertyChanged("Title");
            }
        }
        
        public ObservableCollection<IOrganizationUnitViewModel> Children { get; private set; }
        
        public void OnNext(IPerson value)
        {
            if (value.Id != _person.Id) 
                return;
            
            _person = new PersonWrapper(value);
            Title = _person.DisplayName;
        }

        public void OnError(Exception error)
        {
        }

        public void OnCompleted()
        {
        }
    }
}
