﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace Ascon.Pilot.SDK.TaskSample.TaskChat
{
    class TaskChatViewModel : INotifyPropertyChanged, IObserver<IDataObject>, IObserver<ITaskMessage>
    {
        protected readonly ITaskModifier _taskModifier;
        protected readonly IObjectsRepository _repository;
        private readonly IPilotDialogService _pilotDialogService;
        private readonly ITaskObject _task;
        private ObservableCollection<ITaskMessage> _messages = new ObservableCollection<ITaskMessage>();
        private string _newMessageText;

        public TaskChatViewModel( ITaskObject task, ITaskModifier taskModifier, IObjectsRepository repository, IPilotDialogService pilotDialogService)
        {
            if (task == null) 
                throw new ArgumentNullException("task");

            if (taskModifier == null)
                throw new ArgumentNullException("taskModifier");

            if (repository == null)
                throw new ArgumentNullException("repository");

            if (pilotDialogService == null)
                throw new ArgumentNullException("pilotDialogService");

            _task = task;
            _taskModifier = taskModifier;
            _repository = repository;
            _pilotDialogService = pilotDialogService;

            _repository.SubscribeObjects(new[] {_task.ChatId}).Subscribe(this);
        }

        public ObservableCollection<ITaskMessage> Messages
        {
            get { return _messages; }
        }

        public string NewMessageText
        {
            get { return _newMessageText; }
            set
            {
                _newMessageText = value;
                OnPropertyChanged("NewMessageText");
            }
        }

        public void SendNewMessage()
        {
            if(string.IsNullOrEmpty(NewMessageText))
                return;

            MessageBox.Show(NewMessageText);
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

        public void OnNext(IDataObject value)
        {
            var taskChatObject = value;
            if (taskChatObject.Children.Count == 0)
            {
                MessageBox.Show("This task does not have messages in chat yet.");
                return;
            }

            _repository.SubscribeTaskMessages(taskChatObject.Children).Subscribe(this);
        }

        public void OnNext(ITaskMessage value)
        {
            _messages.Add(value);
        }

        void IObserver<ITaskMessage>.OnError(Exception error)
        {
 
        }

        void IObserver<ITaskMessage>.OnCompleted()
        {
           
        }

        void IObserver<IDataObject>.OnError(Exception error)
        {
           
        }

        void IObserver<IDataObject>.OnCompleted()
        {
            
        }
    }
}
