﻿using System.Linq;

namespace Ascon.Pilot.SDK.TaskSample.TaskEditView
{
    class AgreementTaskEditViewModel : TaskEditViewModel
    {
        public AgreementTaskEditViewModel(ITaskModifier taskModifier, IObjectsRepository repository, IPilotDialogService pilotDialogService, ITabServiceProvider tabServiceProvider) 
            : base(taskModifier, repository, pilotDialogService, tabServiceProvider)
        {
        }

        protected override void UpdateCanSubmit()
        {
            CanSubmit = (!string.IsNullOrEmpty(Title) && _executorItems.Any(x => x.OrgUnitId != -1) && _attachments.Any());
        }

        public override bool SubmitTask()
        {
            if (!CanSubmit)
                return false;

            var attachments = Attachments.Select(x => x.Id);
            var workflow = _taskModifier.NewWorkflow().SetTitle(Title).SetDescription(Description).SetInitiatorAttachments(attachments.ToList());
            var stage = _taskModifier.NewStage(workflow.WorkflowObject).SetStageOrder(1);
            var executors = _executorItems.Where(x => x.OrgUnitId != -1).Select(x => x.OrgUnitId).ToList();
            var auditorIds = _auditorsItems.Where(a => a.OrgUnitId != -1).Select(a => a.OrgUnitId).ToList();
            foreach (var executor in executors)
            {
                var taskBuilder = _taskModifier.NewAgreementTask(workflow.WorkflowObject, stage.StageObject, executor).SetExecutorRole("Check");
                taskBuilder.SetAuditors(auditorIds);

            }
            _taskModifier.Apply();

            return true;
        }
    }
}
