﻿using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Media;
using Ascon.Pilot.Theme.ColorScheme;

namespace Ascon.Pilot.SDK.WPFSample
{
    [Export(typeof(INewTabPage))]
    public class OpenSpacePlugin : INewTabPage
    {
        private readonly ITabServiceProvider _tabServiceProvider;
        private const string CommandName = "WPFControlsExtension";
        private const string Title = "WPF controls extension";

        private const string SingleCommandName = "SingleWPFControlsExtension";
        private const string SingleTitle = "Single WPF controls extension";

        private const string OpenInOtherPaneCommandName = "WPFControlsExtensionInOtherPane";
        private const string OpenInOtherPaneTitle = "Open WPF controls extension in other pane";

        private const string CloseThisCommandName = "WPFControlsExtensionClose";
        private const string CloseThisTitle = "Close this tab page extension";

        private const string OpenNewTabCommandName = "WPFControlsExtensionOpenNewTab";
        private const string OpenNewTabTitle = "Open new tab page extension";

        [ImportingConstructor]
        public OpenSpacePlugin(ITabServiceProvider tabServiceProvider, IPilotDialogService dialogService)
        {
            _tabServiceProvider = tabServiceProvider;
            var accentColor = (Color)ColorConverter.ConvertFromString(dialogService.AccentColor);
            ColorScheme.Initialize(accentColor, dialogService.Theme);
        }

        public void BuildNewTabPage(INewTabPageHost host)
        {
            var toolTip = "Open tab with WPF Controls extension";
            var icon = IconLoader.GetIcon("/Resources/plugin_icon.svg");
            host.AddButton(Title, CommandName, toolTip, icon);
            toolTip = "Open tab with single WPF controls extension";
            host.AddButton(SingleTitle, SingleCommandName, toolTip, icon);
            toolTip = "Open WPF controls extension in other pane";
            host.AddButton(OpenInOtherPaneTitle, OpenInOtherPaneCommandName, toolTip, icon);
            toolTip = "Open new tab page extension";
            host.AddButton(OpenNewTabTitle, OpenNewTabCommandName, toolTip, icon);
            toolTip = "Close this tab page extension";
            host.AddButton(CloseThisTitle, CloseThisCommandName, toolTip, icon);
        }

        public void OnButtonClick(string name)
        {
            var activeTabPageTitle = _tabServiceProvider.GetActiveTabPageTitle();

            if (name == CommandName)
            {
                var view = new MainView();
                _tabServiceProvider.UpdateTabPageContent(activeTabPageTitle, Title, view);
                return;
            }

            if (name == SingleCommandName)
            {
                OpenSingleExtensionTabPage(activeTabPageTitle);
                return;
            }

            if (name == OpenInOtherPaneCommandName)
            {
                var view = new MainView();
                _tabServiceProvider.OpenTabPage(OpenInOtherPaneTitle, view, true);
                return;
            }

            if (name == CloseThisCommandName)
            {
                var activeTabPage = _tabServiceProvider.GetActiveTabPageTitle();
                _tabServiceProvider.CloseTabPage(activeTabPage);
                return;
            }

            if (name == OpenNewTabCommandName)
            {
                _tabServiceProvider.OpenNewTabPage();
            }
        }

        private void OpenSingleExtensionTabPage(string activeTabPageTitle)
        {
            var titles = _tabServiceProvider.GetTabPageTitles();
            if (titles.Contains(SingleTitle))
            {
                //close new tab page
                var activeTabPage = _tabServiceProvider.GetActiveTabPageTitle();
                _tabServiceProvider.CloseTabPage(activeTabPage);
                //activate single extension tab page
                _tabServiceProvider.ActivateTabPage(SingleTitle);
                return;
            }

            var singleView = new MainView();
            _tabServiceProvider.UpdateTabPageContent(activeTabPageTitle, SingleTitle, singleView);
        }
    }
}