﻿using System;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;
using System.Xml.Linq;
using Ascon.Pilot.SDK.Menu;
using Ascon.Pilot.SDK.XpsViewerSample.Properties;
using Ascon.Pilot.SDK.CreateObjectSample;


namespace Ascon.Pilot.SDK.XpsViewerSample
{
    [Export(typeof(IMenu<GraphicLayerElementContext>))]
    public class GraphicLayerContextMenuSample : IMenu<GraphicLayerElementContext>
    {
        private readonly IObjectModifier _modifier;
        private readonly IObjectsRepository _repository;
        private readonly IFileProvider _fileProvider;

        [ImportingConstructor]
        public GraphicLayerContextMenuSample(IObjectModifier modifier, IObjectsRepository repository, IFileProvider fileProvider)
        {
            _modifier = modifier;
            _repository = repository;
            _fileProvider = fileProvider;
        }

        public void Build(IMenuBuilder builder, GraphicLayerElementContext context)
        {
            builder.AddItem("miChangeTextInGraphicLayerElement", builder.Count)
                .WithHeader(Resources.EditGraphicLayerElementTitle);
        }

        public void OnMenuItemClick(string name, GraphicLayerElementContext context)
        {
            if (name == "miChangeTextInGraphicLayerElement")
            {
                var loader = new ObjectLoader(_repository);
                loader.Load(context.DataObject.Id, obj =>
                {
                    ChangeGraphicLayerElementText(obj, context.ElementId);
                });
            }
        }

        [DllImport("user32.dll")]
        public static extern IntPtr GetForegroundWindow();

        private void ChangeGraphicLayerElementText(IDataObject dataObject, Guid contextElementId)
        {
            var graphicLayerElement = dataObject.Files.FirstOrDefault(f => f.Name.Contains(contextElementId.ToString()));
            if (graphicLayerElement != null)
            {
                XElement xamlObject;
                if (!TryParseXamlObject(graphicLayerElement, out xamlObject))
                    return;

                var builder = _modifier.Edit(dataObject);
                var viewModel = new EditDialogViewModel(xamlObject.ToString());
                var editView = new EditDialogView { DataContext = viewModel };
                var activeWindowHandle = GetForegroundWindow();
                new WindowInteropHelper(editView)
                {
                    Owner = activeWindowHandle
                };

                if (editView.ShowDialog() == true)
                {
                    xamlObject = XElement.Parse(viewModel.Text);
                    using (var textBlocksStream = new MemoryStream())
                    {
                        using (var writer = new StreamWriter(textBlocksStream))
                        {
                            writer.Write(xamlObject);
                            writer.Flush();
                            builder.AddOrReplaceFile(GraphicLayerElementConstants.GRAPHIC_LAYER_ELEMENT_CONTENT + contextElementId, textBlocksStream, graphicLayerElement, DateTime.Now, DateTime.Now, DateTime.Now);
                        }
                    }
                    _modifier.Apply();
                }
            }
        }

        private bool TryParseXamlObject(IFile graphicLayerElement, out XElement xamlObject)
        {
            xamlObject = null;
            using (var stream = _fileProvider.OpenRead(graphicLayerElement))
            {
                using (var sr = new StreamReader(stream))
                {
                    try
                    {
                        xamlObject = XElement.Parse(sr.ReadToEnd());
                    }
                    catch (System.Xml.XmlException)
                    {
                        MessageBox.Show(Resources.CannotEditElement);
                        return false;
                    }
                }
            }
            return true;
        }
    }
}