﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Threading;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.Menu;
using Ascon.Pilot.Theme.ColorScheme;
using Ascon.Pilot.Theme.Controls;
using ChangeFilesMonitor.Tools;
using WindowState = ChangeFilesMonitor.Tools.WindowState;

namespace ChangeFilesMonitor
{
    [Export(typeof(IMenu<SystemTrayContext>))]
    public class Main : IMenu<SystemTrayContext>, IHandle<UnloadedEventArgs>, IHandle<LoadedEventArgs>
    {
        private readonly IObjectsRepository _repository;
        private readonly IClientInfo _clientInfo;
        private Window _dialog;

        private const string PLUGIN_TITLE = "Измененные файлы";

        [ImportingConstructor]
        public Main(IEventAggregator eventAggregator, IPilotDialogService service, IObjectsRepository repository, IClientInfo clientInfo)
        {
            _repository = repository;
            _clientInfo = clientInfo;
            eventAggregator.Subscribe(this);
            
            InitColorScheme(service);
        }

        private void ShowDialog()
        {
            _dialog = CreateWindow();
            _dialog.ShowDialog();
        }

        private Window CreateWindow()
        {
            var view = new MainView();
            var dialog = new DialogWindow();

            var style = (Style)view.FindResource("DialogWindowStyle");
            dialog.Style = style;

            var storageCommandController = new StorageCommandController(_clientInfo);
            var viewModel = new MainViewModel(_repository, dialog.Dispatcher, storageCommandController);
            view.DataContext = viewModel;
            dialog.Content = view;
            dialog.ShowInTaskbar = true;
            dialog.MinWidth = 150;
            dialog.Width = 250;
            dialog.MinHeight= 200;
            dialog.Height = 400;
            dialog.Topmost = true;
            dialog.Title = PLUGIN_TITLE;

            dialog.Loaded += DialogOnLoaded;
            dialog.Closing += DialogOnClosing;
            return dialog;
        }

        private void DialogOnClosing(object sender, CancelEventArgs cancelEventArgs)
        {
            SaveState(sender);
        }

        private static void SaveState(object sender)
        {
            var settingsPath = SpecialDirectoryProvider.SettingsFilename;
            var settings = new Tools.WindowState();

            var window = (Window) sender;
            settings.Left = window.Left;
            settings.Top = window.Top;
            settings.Width = window.Width;
            settings.Height = window.Height;

            SettingsStore.SaveWindowsPosition(settingsPath, settings);
        }

        private void DialogOnLoaded(object sender, RoutedEventArgs routedEventArgs)
        {
            var settingsPath = SpecialDirectoryProvider.SettingsFilename;
            var settings = SettingsStore.GetWindowsPosition(settingsPath);

            var window = (Window)sender;
            window.Left = settings.Left;
            window.Top = settings.Top;
            window.Width = settings.Width;
            window.Height = settings.Height;
        }

        private void InitColorScheme(IPilotDialogService pilotDialogService)
        {
            var accentColorObj = ColorConverter.ConvertFromString(pilotDialogService.AccentColor);
            if (accentColorObj == null)
                return;

            var accentColor = (Color)accentColorObj;
            ColorScheme.Initialize(accentColor, pilotDialogService.Theme);
        }

        public void Handle(UnloadedEventArgs message)
        {
            _dialog.Dispatcher.BeginInvoke(new Action(() => SaveState(_dialog)));
            _dialog.Dispatcher.BeginInvokeShutdown(DispatcherPriority.ApplicationIdle);
        }

        public void Handle(LoadedEventArgs message)
        {
            RunDialog();
        }

        private void RunDialog()
        {
            var thread = new Thread(ShowDialog);
            thread.SetApartmentState(ApartmentState.STA); //Set the thread to STA
            thread.Start();
            //_thread.Join(); //Wait for the thread to end
        }

        public void Build(IMenuBuilder builder, SystemTrayContext context)
        {
            builder.AddItem("ChangeFilesMonitor", 0).WithHeader(PLUGIN_TITLE);
        }

        public void OnMenuItemClick(string name, SystemTrayContext context)
        {
            RunDialog();
        }
    }
}
