using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using ProtoBuf;

namespace ChangeFilesMonitor.Shell
{
    [ComVisible(false)]
    public enum ShellResult
    {
        Ok,
        Error
    }

    [ComVisible(false)]
    [ProtoContract]
    public class CommandInvokeData
    {
        [ProtoMember(1)]
        public Guid CommandId { get; set; }

        [ProtoMember(2)]
        public List<string> Paths { get; private set; }

        [ProtoMember(3)]
        public string ExtraArgs { get; set; }

        public CommandInvokeData()
        {
            Paths = new List<string>();
        }
    }

    [ComVisible(false)]
    [ProtoContract]
    public class CommandInvokeResult
    {
        [ProtoMember(1)]
        public ShellResult Result { get; set; }

        [ProtoMember(2)]
        public byte[] Data { get; set; }
    }

    [ComVisible(false)]
    [ProtoContract]
    public class MenuData
    {
        [ProtoMember(1)]
        public ShellResult Result { get; set; }

        [ProtoMember(2)]
        public List<MenuItemData> Items { get; private set; }

        public MenuData()
        {
            Items = new List<MenuItemData>();
        }
    }

    [ComVisible(false)]
    [ProtoContract]
    public class MenuItemData
    {
        [ProtoMember(1)]
        public string Header { get; set; }

        [ProtoMember(2)]
        public Guid CommandId { get; set; }

        [ProtoMember(3)]
        public byte[] Icon { get; set; }

        [ProtoMember(4)]
        public bool IsEnabled { get; set; }

        [ProtoMember(5)]
        public List<MenuItemData> Subitems { get; private set; }

        public MenuItemData()
        {
            Subitems = new List<MenuItemData>();
        }
    }

    [ComVisible(false)]
    public static class CommandIds
    {
        public static readonly Guid Commit = new Guid("8458614C-1FE3-4DFA-B2BF-BB2AF8BEF320");
        public static readonly Guid Discard = new Guid("EA1B8F98-AFDC-4328-9C35-B878AD9BAE8B");
        public static readonly Guid Download = new Guid("F0A2A4E1-458B-4B4C-A259-73DC69894278");
        public static readonly Guid GetLatestVersion = new Guid("FE8B7DF7-B785-47D6-950E-121D6D35509C");
        public static readonly Guid Revert = new Guid("246ABBE1-F7AC-4241-B4F4-47E3B95952CE");
        public static readonly Guid SharingSettings = new Guid("4F42CDEA-E3C6-4F95-B731-504CA71829FC");
        public static readonly Guid ShowDocument = new Guid("E3DA0178-03E7-4903-BE67-C2EA73E4753F");
        public static readonly Guid QuitClient = new Guid("966ecbe2-4b5f-464b-9e96-12bc10e93885");
        public static readonly Guid ShowAllPrinters = new Guid("ca895788-2238-481f-8ed4-406d109777bb");
        public static readonly Guid ShowHistory = new Guid("798B2FF1-80F0-4BAD-8C4E-4562466A99AA");
        public static readonly Guid ShowProjectsExplorer = new Guid("12940BB8-E819-43D8-8B1F-9F018C25BA5F");
        public static readonly Guid ShowRecycleBin = new Guid("C47B1458-CE84-407C-B7F6-A60C402275E0");
        public static readonly Guid Subsribe = new Guid("8271C139-A06F-4E51-B5C6-5200E90F6FA8");
        public static readonly Guid Unsubscribe = new Guid("2D559484-9A3A-4374-8C90-E9752DDE5A14");
        public static readonly Guid Unmount = new Guid("5EC20BE2-B47A-4644-8537-FF6A9AAB9517");
        public static readonly Guid BuildContextMenu = new Guid("54808C44-0091-4638-829B-FC574798464B");
        public static readonly Guid Lock = new Guid("BFB8A2C3-B460-43CD-BE97-D97E860CAE72");
        public static readonly Guid Unlock = new Guid("E6F5DE56-E8BE-490C-B79A-CCA6DCFDE380");
        public static readonly Guid AttachToTask = new Guid("56E9F934-1C0D-4F4B-BF8F-8FEA46CA2F21");
        public static readonly Guid AttachToTaskFromTemplate = new Guid("928C714C-3329-496E-8C7E-69CDACD67676");
        public static readonly Guid CopyLink = new Guid("192E13FA-4450-4EFF-9578-1941668DC9CE");
    }
}
