﻿using System;
using System.IO;
using System.Runtime.InteropServices;

namespace ChangeFilesMonitor.Shell
{
    [ComVisible(false)]
    public static class ShellClientConstants
    {
        public const string INFO_PIPE_NAME = "PilotInfoPipe";
        public const string COMMAND_PIPE_NAME = "PilotCommandPipe";
        public const string FILES_SEPARATOR = ">";
        public const string VIRTUAL_DRIVE_FORMAT_NAME = "pfmfs.";
        public const string NTFS_DRIVE_FORMAT_NAME = "NTFS";
        public const string PROTOCOL_VERSION = "20160511";

        public static string GetPilotPipeServerAddress(string path, string pipeName)
        {
            try
            {
                if (string.IsNullOrEmpty(path))
                    return null;

                if (!char.IsLetter(path, 0))
                    return null;

                var root = Path.GetPathRoot(path);
                if (string.IsNullOrEmpty(root))
                    return null;

                var driveInfo = new DriveInfo(root);
                if (driveInfo.DriveType != DriveType.Fixed)
                    return null;

                var driveFormat = driveInfo.DriveFormat;
                return driveFormat.StartsWith(VIRTUAL_DRIVE_FORMAT_NAME)
                    ? pipeName + driveInfo.DriveFormat + PROTOCOL_VERSION
                    : null;
            }
            catch (Exception)
            {
                return null;
            }
        }

        public static bool IsVirtualDrivePath(string path)
        {
            try
            {
                return GetPilotPipeServerAddress(path, string.Empty) != null;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public static bool IsVirtualDrivePathFor(this string path, string clientName)
        {
            try
            {
                var serverName = GetPilotPipeServerAddress(path, string.Empty);
                return serverName != null && serverName.EndsWith(clientName + PROTOCOL_VERSION);
            }
            catch (Exception)
            {
                return false;
            }
        }

        public static string GetVirtualPath(string path)
        {
            return path.Substring(2);
        }

        public static string BuildServerAddress(string serverName, string clientName)
        {
            return string.Format("{0}{1}{2}{3}", serverName, VIRTUAL_DRIVE_FORMAT_NAME, clientName, PROTOCOL_VERSION);
        }
    }
}
