﻿using System;
using Ascon.Pilot.SDK;
using ChangeFilesMonitor.Shell;

namespace ChangeFilesMonitor
{
    class StorageCommandController
    {
        private readonly string _address;

        public StorageCommandController(IClientInfo clientInfo)
        {
            var productName = GetProductName(clientInfo);
            _address = ShellClientConstants.BuildServerAddress(ShellClientConstants.COMMAND_PIPE_NAME, productName);
        }

        public bool Commit(string path)
        {
            var commandData = new CommandInvokeData()
            {
                CommandId = CommandIds.Commit
            };
            commandData.Paths.Add(path);
            var result = ShellClient.Request<CommandInvokeData, CommandInvokeResult>(commandData, _address);
            return result.Result == ShellResult.Ok;
        }

        public PilotStorageIcon GetIcon(string path)
        {
            var icon = ShellClient.RequestIcon(path, _address);
            return icon;
        }

        private string GetProductName(IClientInfo clientInfo)
        {
            var type = clientInfo.GetClientType();
            switch (type)
            {
                case ClientType.Ice:
                    return "Pilot-ICE";
                case ClientType.IceEnterprise:
                    return "Pilot-ICE Enterprise";
                case ClientType.ThreeDStorage:
                    return "3D-Storage";
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }
}
