﻿using System;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interop;

namespace ChangeFilesMonitor.Tools
{
    public sealed class HotKey : IDisposable
    {

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool RegisterHotKey(IntPtr hWnd, int id, int fsModifiers, uint vk);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool UnregisterHotKey(IntPtr hWnd, int id);

        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        public static extern IntPtr GetDesktopWindow();

        private const int WM_HOT_KEY = 0x0312;

        public event Action<HotKey> HotKeyPressed;

        private readonly int _id;
        private bool _isKeyRegistered;
        private readonly IntPtr _handle;
        private readonly Keys _key;
        private readonly ModifierKeys _keyModifier;

        public HotKey(ModifierKeys modifierKeys, Keys key, Window window)
            : this(modifierKeys, key, new WindowInteropHelper(window))
        {
        }

        public HotKey(ModifierKeys modifierKeys, Keys key, WindowInteropHelper window)
            : this(modifierKeys, key, window.Handle)
        {
        }

        public HotKey(ModifierKeys modifierKeys, Keys key, IntPtr windowHandle)
        {
            _key = key;
            _keyModifier = modifierKeys;
            _id = GetHashCode();
            _handle = windowHandle;
            RegisterHotKey();
            ComponentDispatcher.ThreadPreprocessMessage += ThreadPreprocessMessageMethod;
        }

        public Keys Key
        {
            get { return _key; }
        }

        public ModifierKeys KeyModifier
        {
            get { return _keyModifier; }
        }

        public void Dispose()
        {
            ComponentDispatcher.ThreadPreprocessMessage -= ThreadPreprocessMessageMethod;
            UnregisterHotKey();
        }

        private void RegisterHotKey()
        {
            if (_key == Keys.None)
                return;            
            
            if (_isKeyRegistered)
                UnregisterHotKey();

            RegisterHotKey(_handle, _id, (int)KeyModifier, (uint)_key);
        }

        private void UnregisterHotKey()
        {
            _isKeyRegistered = !UnregisterHotKey(_handle, _id);
        }

        private void ThreadPreprocessMessageMethod(ref MSG msg, ref bool handled)
        {
            if (handled) 
                return;

            if (msg.message != WM_HOT_KEY || (int) (msg.wParam) != _id) 
                return;
            
            OnHotKeyPressed();
            handled = true;
        }

        private void OnHotKeyPressed()
        {
            if (HotKeyPressed != null)
                HotKeyPressed(this);
        }
    }
}
