﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.Menu;
using ProjectCloneWizard.Common;

namespace ProjectCloneWizard
{
    [Export(typeof(IMenu<ObjectsViewContext>))]
    public class ProjectCloneMenu : IMenu<ObjectsViewContext>
    {
        private readonly ProjectCloneCreator _projectCloneCreator;

        [ImportingConstructor]
        public ProjectCloneMenu(IObjectsRepository repository, IPilotDialogService pilotDialogService, IAttributeFormatParser attributeFormatParser, 
            IObjectModifier modifier, IFileProvider fileProvider, ITabServiceProvider tabServiceProvider)
        {
            _projectCloneCreator = new ProjectCloneCreator(repository, pilotDialogService, attributeFormatParser, modifier, fileProvider, tabServiceProvider);
        }

        public void Build(IMenuBuilder builder, ObjectsViewContext context)
        {
            var objects = context.SelectedObjects.ToList();
            if (objects.Count != 1)
                return;

            var menu = builder.ItemNames.ToList();
            if (!menu.Contains("miCreate"))
                return;

            var subMenu = builder.GetItem("miCreate");
            var index = subMenu.Count;
            
            var smartFolderIndex = subMenu.ItemNames.ToList().IndexOf("miCreateSmartfoldertype");
            if (smartFolderIndex != -1)
                index = smartFolderIndex - 1;

            var selected = objects.First();
            if (selected.Type.IsProject)
            {
                subMenu.AddItem(Const.COMMAND_UPDATE_NAME, index)
                    .WithHeader(Const.COMMAND_UPDATE_TITLE)
                    .WithIcon(IconLoader.GetIcon(_projectCloneCreator.Icon));
            }
            else
            {
                subMenu.AddItem(Const.COMMAND_NAME, index)
                    .WithHeader(Const.COMMAND_TITLE)
                    .WithIcon(IconLoader.GetIcon(_projectCloneCreator.Icon));
            }
        }

        public void OnMenuItemClick(string name, ObjectsViewContext context)
        {
            var selected = context.SelectedObjects.First();
            _projectCloneCreator.ShowWizardDialog(selected);
        }
    }
}
